///
/// EarthBrowser
///
/// Copyright (c) 2008 Lunar Software, Inc.
///

/*
Copyright 2006 Adobe Systems Incorporated

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.


THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/


/*
 * The Bridge class, responsible for navigating AS instances
 */
function FABridge(target,bridgeName)
{
	this.target = target;
	this.remoteTypeCache = {};
	this.remoteInstanceCache = {};
	this.remoteFunctionCache = {};
	this.localFunctionCache = {};
	this.bridgeID = FABridge.nextBridgeID++;
	this.name = bridgeName;
	this.nextLocalFuncID = 0;
	FABridge.instances[this.name] = this;
	FABridge.idMap[this.bridgeID] = this;

	return this;
}

// type codes for packed values
FABridge.TYPE_ASINSTANCE =  1;
FABridge.TYPE_ASFUNCTION =  2;

FABridge.TYPE_JSFUNCTION =  3;
FABridge.TYPE_ANONYMOUS =   4;

FABridge.initCallbacks = {};
FABridge.userTypes = {};

FABridge.addToUserTypes = function()
{
	for (var i = 0; i < arguments.length; i++)
	{
		FABridge.userTypes[arguments[i]] = {
			'typeName': arguments[i], 
			'enriched': false
		};
	}
}

FABridge.argsToArray = function(args)
{
	var result = [];
	for (var i = 0; i < args.length; i++)
	{
		result[i] = args[i];
	}
	return result;
}

function instanceFactory(objID)
{
	this.fb_instance_id = objID;
	return this;
}

function FABridge__invokeJSFunction(args)
{  
	var funcID = args[0];
	var throughArgs = args.concat();//FABridge.argsToArray(arguments);
	throughArgs.shift();
	
   
	var bridge = FABridge.extractBridgeFromID(funcID);
	return bridge.invokeLocalFunction(funcID, throughArgs);
}

FABridge.addInitializationCallback = function(bridgeName, callback)
{
	var inst = FABridge.instances[bridgeName];
	if (inst != undefined)
	{
		callback.call(inst);
		return;
	}

	var callbackList = FABridge.initCallbacks[bridgeName];
	if(callbackList == null)
	{
		FABridge.initCallbacks[bridgeName] = callbackList = [];
	}

	callbackList.push(callback);
}

/**
 * attach the bridge to an instance of a flash movie on the page
 * by looking at the flashvars to see if the bridgeName is present
 * called from AS when the bridge is initialized to attach the JS
 */
/*
function FABridge__bridgeInitialized(bridgeName)
{
	var searchStr = "bridgeName="+ bridgeName;
	
	///
	var userAgent = navigator.userAgent.toLowerCase();

	/// Figure out what browser is being used
	var browser = {
		version: (userAgent.match(/.+(?:rv|it|ra|ie)[\/: ]([\d.]+)/) || [])[1],
		safari: /webkit/.test(userAgent),
		opera: /opera/.test(userAgent),
		msie: /msie/.test(userAgent) && !/opera/.test(userAgent),
		mozilla: /mozilla/.test(userAgent) && !/(compatible|webkit)/.test(userAgent)
	};
	
//	if (/Explorer/.test(navigator.appName) || /Konqueror|Safari|KHTML/.test(navigator.appVersion))
	if(browser.msie || browser.safari || browser.mozilla)
	{
		var flashInstances = document.getElementsByTagName("object");
		if (flashInstances.length == 1)
		{
			FABridge.attachBridge(flashInstances[0], bridgeName);
		}
		else
		{
			for(var i = 0; i < flashInstances.length; i++)
			{
				var inst = flashInstances[i];
				var params = inst.childNodes;
				var flash_found = false;

				for (var j = 0; j < params.length; j++)
				{
					var param = params[j];
					if (param.nodeType == 1 && param.tagName.toLowerCase() == "param")
					{
						if (param["name"].toLowerCase() == "flashvars" && param["value"].indexOf(searchStr) >= 0)
						{
							FABridge.attachBridge(inst, bridgeName);
							flash_found = true;
							break;
						}
					}
				}

				if (flash_found) {
					break;
				}
			}
		}
	}
	else
	{
		var flashInstances = document.getElementsByTagName("embed");
		if (flashInstances.length == 1)
		{
			FABridge.attachBridge(flashInstances[0], bridgeName);
		}
		else
		{
			for(var i = 0; i < flashInstances.length; i++)
			{
				var inst = flashInstances[i];
				var flashVars = inst.attributes.getNamedItem("flashVars").nodeValue;
				if (flashVars.indexOf(searchStr) >= 0)
				{
					FABridge.attachBridge(inst, bridgeName);
				}

			}
		}
	}
	return true;
}
*/

function FABridge__setStaticInstance(inst) {
	FABridge.staticInstance = inst;
}
function FABridge__bridgeInitialized(bridgeName) {
	air.trace('FABridge__bridgeInitialized(' + bridgeName + ') ');
	if(FABridge.staticInstance != null)
	{
		FABridge.attachBridge(FABridge.staticInstance, bridgeName);
		return true;
	}
	
    var objects = document.getElementsByTagName("object");
    var ol = objects.length;
    var activeObjects = [];
    if (ol > 0) {
		for (var i = 0; i < ol; i++) {
			if (typeof objects[i].SetVariable != "undefined") {
				activeObjects[activeObjects.length] = objects[i];
			}
		}
	}
    var embeds = document.getElementsByTagName("embed");
    var el = embeds.length;
    var activeEmbeds = [];
    if (el > 0) {
		for (var j = 0; j < el; j++) {
			if (typeof embeds[j].SetVariable != "undefined") {
            	activeEmbeds[activeEmbeds.length] = embeds[j];
            }
        }
    }
    var aol = activeObjects.length;
    var ael = activeEmbeds.length;
    var searchStr = "bridgeName="+ bridgeName;
    if ((aol == 1 && !ael) || (aol == 1 && ael == 1)) {
    	FABridge.attachBridge(activeObjects[0], bridgeName);	 
    }
    else if (ael == 1 && !aol) {
    	FABridge.attachBridge(activeEmbeds[0], bridgeName);
    }
    else {
    	var flash_found = false;
		if (aol > 1) {
			for (var k = 0; k < aol; k++) {
				 var params = activeObjects[k].childNodes;
				 for (var l = 0; l < params.length; l++) {
					var param = params[l];
					if (param.nodeType == 1 && param.tagName.toLowerCase() == "param" && param["name"].toLowerCase() == "flashvars" && param["value"].indexOf(searchStr) >= 0) {
						FABridge.attachBridge(activeObjects[k], bridgeName);
						flash_found = true;
						break;
					}
				}
				if (flash_found) {
					break;
				}
			}
		}
		if (!flash_found && ael > 1) {
			for (var m = 0; m < ael; m++) {
				var flashVars = activeEmbeds[m].attributes.getNamedItem("flashVars").nodeValue;
				if (flashVars.indexOf(searchStr) >= 0) {
					FABridge.attachBridge(activeEmbeds[m], bridgeName);
					break;
				}
			}
		}
	}
    return true;
}


// used to track multiple bridge instances, since callbacks from AS are global across the page.

FABridge.nextBridgeID = 0;
FABridge.instances = {};
FABridge.idMap = {};
FABridge.refCount = 0;
FABridge.staticInstance = null;

FABridge.extractBridgeFromID = function(id)
{
	var bridgeID = (id >> 16);
	return FABridge.idMap[bridgeID];
}

FABridge.attachBridge = function(instance, bridgeName)
{
	var newBridgeInstance = new FABridge(instance, bridgeName);

	FABridge[bridgeName] = newBridgeInstance;

/*  FABridge[bridgeName] = function() {
		return newBridgeInstance.root();
	}
*/
	var callbacks = FABridge.initCallbacks[bridgeName];
	if (callbacks == null)
	{
		return;
	}
	for (var i = 0; i < callbacks.length; i++)
	{
		callbacks[i].call(newBridgeInstance);
	}
	delete FABridge.initCallbacks[bridgeName]
}

// some methods can't be proxied.  You can use the explicit get,set, and call methods if necessary.

FABridge.blockedMethods =
{
	toString: true,
	get: true,
	set: true,
	call: true
};

FABridge.prototype =
{


// bootstrapping

	root: function()
	{
		return this.deserialize(this.target.getRoot());
	},
//clears all of the AS objects in the cache maps
	releaseASObjects: function()
	{
		return this.target.releaseASObjects();
	},
//clears a specific object in AS from the type maps
	releaseNamedASObject: function(value)
	{
		if(typeof(value) != "object")
		{
			return false;
		}
		else
		{
			var ret =  this.target.releaseNamedASObject(value.fb_instance_id);
			return ret;
		}
	},
//create a new AS Object
	create: function(className)
	{
		return this.deserialize(this.target.create(className));
	},


	// utilities

	makeID: function(token)
	{
		return (this.bridgeID << 16) + token;
	},


	// low level access to the flash object

//get a named property from an AS object
	getPropertyFromAS: function(objRef, propName)
	{
		if (FABridge.refCount > 0)
		{
			throw new Error("You are trying to call recursively into the Flash Player which is not allowed. In most cases the JavaScript setTimeout function, can be used as a workaround.");
		}
		else
		{
			FABridge.refCount++;
			retVal = this.target.getPropFromAS(objRef, propName);
			retVal = this.handleError(retVal);
			FABridge.refCount--;
			return retVal;
		}
	},
//set a named property on an AS object
	setPropertyInAS: function(objRef,propName, value)
	{
		if (FABridge.refCount > 0)
		{
			throw new Error("You are trying to call recursively into the Flash Player which is not allowed. In most cases the JavaScript setTimeout function, can be used as a workaround.");
		}
		else
		{
			FABridge.refCount++;
			retVal = this.target.setPropInAS(objRef,propName, this.serialize(value));
			retVal = this.handleError(retVal);
			FABridge.refCount--;
			return retVal;
		}
	},

//call an AS function
	callASFunction: function(funcID, args)
	{
		if (FABridge.refCount > 0)
		{
			throw new Error("You are trying to call recursively into the Flash Player which is not allowed. In most cases the JavaScript setTimeout function, can be used as a workaround.");
		}
		else
		{
			FABridge.refCount++;
			retVal = this.target.invokeASFunction(funcID, this.serialize(args));
			retVal = this.handleError(retVal);
			FABridge.refCount--;
			return retVal;
		}
	},
//call a method on an AS object
	callASMethod: function(objID, funcName, args)
	{
		if (FABridge.refCount > 0)
		{
			throw new Error("You are trying to call recursively into the Flash Player which is not allowed. In most cases the JavaScript setTimeout function, can be used as a workaround.");
		}
		else
		{
			FABridge.refCount++;
			args = this.serialize(args);
			retVal = this.target.invokeASMethod(objID, funcName, args);
			retVal = this.handleError(retVal);
			FABridge.refCount--;
			return retVal;
		}
	},

	// responders to remote calls from flash

	//callback from flash that executes a local JS function
	//used mostly when setting js functions as callbacks on events
	invokeLocalFunction: function(funcID, args)
	{
		var result;
		var func = this.localFunctionCache[funcID];

		if(func != undefined)
		{
			result = this.serialize(func.apply(null, this.deserialize(args)));
		}

		return result;
	},

	// Object Types and Proxies
	getUserTypeDescriptor: function(objTypeName)
	{
		var simpleType = objTypeName.replace(/^([^:]*)\:\:([^:]*)$/, "$2");
		var isUserProto = ((typeof window[simpleType] == "function") && (typeof FABridge.userTypes[simpleType] != "undefined"));

		var protoEnriched = false;
		
		if (isUserProto) {
			protoEnriched = FABridge.userTypes[simpleType].enriched;
		}
		var toret = {
			'simpleType': simpleType, 
			'isUserProto': isUserProto, 
			'protoEnriched': protoEnriched
		};
		return toret;
	}, 
	
	// accepts an object reference, returns a type object matching the obj reference.
	getTypeFromName: function(objTypeName)
	{
		var ut = this.getUserTypeDescriptor(objTypeName);
		var toret = this.remoteTypeCache[objTypeName];
		if (ut.isUserProto)
		{
			//enrich both of the prototypes: the FABridge one, as well as the class in the page. 
			if (!ut.protoEnriched)
			{

				for (i in window[ut.simpleType].prototype)
				{
					toret[i] = window[ut.simpleType].prototype[i];
				}
				
				window[ut.simpleType].prototype = toret;
				this.remoteTypeCache[objTypeName] = toret;
				FABridge.userTypes[ut.simpleType].enriched = true;
			}
		}
		return toret;
	},
	//create an AS proxy for the given object ID and type
	createProxy: function(objID, typeName)
	{
		//get user created type, if it exists
		var ut = this.getUserTypeDescriptor(typeName);

		var objType = this.getTypeFromName(typeName);

		if (ut.isUserProto)
		{
			var instFactory = window[ut.simpleType];
			var instance = new instFactory(this.name, objID);
			instance.fb_instance_id = objID;
		}
		else
		{
			instanceFactory.prototype = objType;
			var instance = new instanceFactory(objID);
		}

		this.remoteInstanceCache[objID] = instance;
		return instance;
	},
	//return the proxy associated with the given object ID
	getProxy: function(objID)
	{
		return this.remoteInstanceCache[objID];
	},

	// accepts a type structure, returns a constructed type
	addTypeDataToCache: function(typeData)
	{
		newType = new ASProxy(this, typeData.name);
		var accessors = typeData.accessors;
		for (var i = 0; i < accessors.length; i++)
		{
			this.addPropertyToType(newType, accessors[i]);
		}

		var methods = typeData.methods;
		for (var i = 0; i < methods.length; i++)
		{
			if (FABridge.blockedMethods[methods[i]] == undefined)
			{
				this.addMethodToType(newType, methods[i]);
			}
		}


		this.remoteTypeCache[newType.typeName] = newType;
		return newType;
	},

	//add a property to a typename; used to define the properties that can be called on an AS proxied object
	addPropertyToType: function(ty, propName)
	{
		var c = propName.charAt(0);
		var setterName;
		var getterName;
		if(c >= "a" && c <= "z")
		{
			getterName = "get" + c.toUpperCase() + propName.substr(1);
			setterName = "set" + c.toUpperCase() + propName.substr(1);
		}
		else
		{
			getterName = "get" + propName;
			setterName = "set" + propName;
		}
		ty[setterName] = function(val)
		{
			this.bridge.setPropertyInAS(this.fb_instance_id, propName, val);
		}
		ty[getterName] = function()
		{
			return this.bridge.deserialize(this.bridge.getPropertyFromAS(this.fb_instance_id, propName));
		}
	},

	//add a method to a typename; used to define the methods that can be callefd on an AS proxied object
	addMethodToType: function(ty, methodName)
	{
		ty[methodName] = function()
		{
			return this.bridge.deserialize(this.bridge.callASMethod(this.fb_instance_id, methodName, FABridge.argsToArray(arguments)));
		}
	},

	// Function Proxies

	//returns the AS proxy for the specified function ID
	getFunctionProxy: function(funcID)
	{
		var bridge = this;
		if (this.remoteFunctionCache[funcID] == null)
		{
			this.remoteFunctionCache[funcID] = function()
			{
				bridge.callASFunction(funcID, FABridge.argsToArray(arguments));
			}
		}
		return this.remoteFunctionCache[funcID];
	},
	
	//reutrns the ID of the given function; if it doesnt exist it is created and added to the local cache
	getFunctionID: function(func)
	{
		if (func.__bridge_id__ == undefined)
		{
			func.__bridge_id__ = this.makeID(this.nextLocalFuncID++);
			this.localFunctionCache[func.__bridge_id__] = func;
		}
		return func.__bridge_id__;
	},

	// serialization / deserialization

	serialize: function(value)
	{
		var result = {};

		var t = typeof(value);
		//primitives are kept as such
		if (t == "number" || t == "string" || t == "boolean" || t == null || t == undefined)
		{
			result = value;
		}
		else if (value instanceof Array)
		{
			//arrays are serializesd recursively
			result = [];
			for (var i = 0; i < value.length; i++)
			{
				result[i] = this.serialize(value[i]);
			}
		}
		else if (t == "function")
		{
			//js functions are assigned an ID and stored in the local cache 
			result.type = FABridge.TYPE_JSFUNCTION;
			result.value = this.getFunctionID(value);
		}
		else if (value instanceof ASProxy)
		{
			result.type = FABridge.TYPE_ASINSTANCE;
			result.value = value.fb_instance_id;
		}
		else
		{
			result.type = FABridge.TYPE_ANONYMOUS;
			result.value = value;
		}

		return result;
	},

	//on deserialization we always check the return for the specific error code that is used to marshall NPE's into JS errors
	// the unpacking is done by returning the value on each pachet for objects/arrays 
	deserialize: function(packedValue)
	{

		var result;

		var t = typeof(packedValue);
		if (t == "number" || t == "string" || t == "boolean" || packedValue == null || packedValue == undefined)
		{
			result = this.handleError(packedValue);
		}
		else if (packedValue instanceof Array)
		{
			result = [];
			for (var i = 0; i < packedValue.length; i++)
			{
				result[i] = this.deserialize(packedValue[i]);
			}
		}
		else if (t == "object")
		{
			for(var i = 0; i < packedValue.newTypes.length; i++)
			{
				this.addTypeDataToCache(packedValue.newTypes[i]);
			}
			for (var aRefID in packedValue.newRefs)
			{
				this.createProxy(aRefID, packedValue.newRefs[aRefID]);
			}
			if (packedValue.type == FABridge.TYPE_PRIMITIVE)
			{
				result = packedValue.value;
			}
			else if (packedValue.type == FABridge.TYPE_ASFUNCTION)
			{
				result = this.getFunctionProxy(packedValue.value);
			}
			else if (packedValue.type == FABridge.TYPE_ASINSTANCE)
			{
				result = this.getProxy(packedValue.value);
			}
			else if (packedValue.type == FABridge.TYPE_ANONYMOUS)
			{
				result = packedValue.value;
			}
		}
		return result;
	},
	//increases the reference count for the given object
	addRef: function(obj)
	{
		this.target.incRef(obj.fb_instance_id);
	},
	//decrease the reference count for the given object and release it if needed
	release:function(obj)
	{
		this.target.releaseRef(obj.fb_instance_id);
	},

	// check the given value for the components of the hard-coded error code : __FLASHERROR
	// used to marshall NPE's into flash
	
	handleError: function(value)
	{
		if (typeof(value)=="string" && value.indexOf("__FLASHERROR")==0)
		{
			var myErrorMessage = value.split("||");
			if(FABridge.refCount > 0 )
			{
				FABridge.refCount--;
			}
			throw new Error(myErrorMessage[1]);
			return value;
		}
		else
		{
			return value;
		}   
	}
};

// The root ASProxy class that facades a flash object

ASProxy = function(bridge, typeName)
{
	this.bridge = bridge;
	this.typeName = typeName;
	return this;
};
//methods available on each ASProxy object
ASProxy.prototype =
{
	get: function(propName)
	{
		return this.bridge.deserialize(this.bridge.getPropertyFromAS(this.fb_instance_id, propName));
	},

	set: function(propName, value)
	{
		this.bridge.setPropertyInAS(this.fb_instance_id, propName, value);
	},

	call: function(funcName, args)
	{
		this.bridge.callASMethod(this.fb_instance_id, funcName, args);
	}, 
	
	addRef: function() {
		this.bridge.addRef(this);
	}, 
	
	release: function() {
		this.bridge.release(this);
	}
};


if(typeof HURLANT=="undefined"||!HURLANT){var HURLANT={};}

HURLANT.jsobject = function(){
	var refs = {};
	var idcount=0;
	var flo = null;

	function genId(obj) {
	  // heavy.
	  for (var a in refs) {
	  	if (refs[a]===obj)
	  		return a;
	  }
	  var id = "obj"+(idcount++);
	//  alert("new ID="+id+" for "+obj)
	  refs[id] = obj;
	  return id;
	}
	
	function resolve(o) {
		if (o==null) return o;
		switch(o.type) {
			case "ref":
				var val = refs[o.id];
				break;
			case "func":
				var val = resolveFunction(o.id);
				break;
			case "value":
				var val = o.val;
				break;
		}
		return val;
	}
	function resolveFunction(id) {
		var o = flo;
		var f = function() {
			var a=arguments;
			var b = [];
			for (var i=0;i<a.length;i++) {
				b[i] = returnRef(a[0]);
			}
			var v = o.call(id, returnRef(this), b);
			return resolve(v);
		};
		return f;
	}
	function returnRef(obj) {
	  var t = typeof obj;
	  if (t=="number"||t=="string"||t=="boolean"||t==null) {
	  	return {value:obj};
	  }
	  var id = genId(obj);
	  if (t=="function") {
		return {id:id, callable:true};
	  } else {
		return {id:id};
	  }
	}
	
	function callRefByValue(name, parent, args) {
		parent = refs[parent];
		var obj = parent[name].apply(parent, args);
		return returnRef(obj);
	};
	
	HURLANT_jsobject_setup = function setup(id) {
			flo = document.getElementById(id);
	};
	HURLANT_jsobject_getReference = function getReference(name, parent) {
	  if (parent==null) { 
	    parent=window;
	  } else {
	    parent = refs[parent];
	  }
	  var obj = parent[name];
	  return returnRef(obj);
	};
	HURLANT_jsobject_setRef = function setRef(name, parent, o) {
		parent = refs[parent];
		parent[name] = resolve(o);
	};
	HURLANT_jsobject_callRefById = function callRefById(name, parent, args) {
		for (var i=0;i<args.length;i++) {
			var a = args[i];
			if (typeof a == "string") {
				args[i] = refs[a];
			} else {
				args[i] = a[0];
			}
		}
		return callRefByValue(name, parent, args);
	};
	HURLANT_jsobject_callRefByValue = callRefByValue;
	HURLANT_jsobject_delReference = function delReference(name, parent) {
		parent = refs[parent];
		try {
			return delete parent[name];
		} catch (e) {
			return false;
		}
	};
	HURLANT_jsobject_hasReference = function hasReference(name, parent) {
		parent = refs[parent];
		return (typeof parent[name] != "undefined"); // not 100% correct, but close enough.
	};
	HURLANT_jsobject_getIndexes = function getIndexes(parent) {
		parent = refs[parent];
		var items = [];
		for (var x in parent) {
			items.push(x);
		}
		return items;
	};
	HURLANT_jsobject_callRef = function callRef(ref, args) {
		ref = refs[ref];
		ref.apply(this, args);
	};
}();

/*
    http://www.JSON.org/json2.js
    2008-09-01

    Public Domain.

    NO WARRANTY EXPRESSED OR IMPLIED. USE AT YOUR OWN RISK.

    See http://www.JSON.org/js.html

    This file creates a global JSON object containing two methods: stringify
    and parse.

        JSON.stringify(value, replacer, space)
            value       any JavaScript value, usually an object or array.

            replacer    an optional parameter that determines how object
                        values are stringified for objects. It can be a
                        function or an array of strings.

            space       an optional parameter that specifies the indentation
                        of nested structures. If it is omitted, the text will
                        be packed without extra whitespace. If it is a number,
                        it will specify the number of spaces to indent at each
                        level. If it is a string (such as '\t' or '&nbsp;'),
                        it contains the characters used to indent at each level.

            This method produces a JSON text from a JavaScript value.

            When an object value is found, if the object contains a toJSON
            method, its toJSON method will be called and the result will be
            stringified. A toJSON method does not serialize: it returns the
            value represented by the name/value pair that should be serialized,
            or undefined if nothing should be serialized. The toJSON method
            will be passed the key associated with the value, and this will be
            bound to the object holding the key.

            For example, this would serialize Dates as ISO strings.

                Date.prototype.toJSON = function (key) {
                    function f(n) {
                        // Format integers to have at least two digits.
                        return n < 10 ? '0' + n : n;
                    }

                    return this.getUTCFullYear()   + '-' +
                         f(this.getUTCMonth() + 1) + '-' +
                         f(this.getUTCDate())      + 'T' +
                         f(this.getUTCHours())     + ':' +
                         f(this.getUTCMinutes())   + ':' +
                         f(this.getUTCSeconds())   + 'Z';
                };

            You can provide an optional replacer method. It will be passed the
            key and value of each member, with this bound to the containing
            object. The value that is returned from your method will be
            serialized. If your method returns undefined, then the member will
            be excluded from the serialization.

            If the replacer parameter is an array of strings, then it will be used to
            select the members to be serialized. It filters the results such
            that only members with keys listed in the replacer array are
            stringified.

            Values that do not have JSON representations, such as undefined or
            functions, will not be serialized. Such values in objects will be
            dropped; in arrays they will be replaced with null. You can use
            a replacer function to replace those with JSON values.
            JSON.stringify(undefined) returns undefined.

            The optional space parameter produces a stringification of the
            value that is filled with line breaks and indentation to make it
            easier to read.

            If the space parameter is a non-empty string, then that string will
            be used for indentation. If the space parameter is a number, then
            the indentation will be that many spaces.

            Example:

            text = JSON.stringify(['e', {pluribus: 'unum'}]);
            // text is '["e",{"pluribus":"unum"}]'


            text = JSON.stringify(['e', {pluribus: 'unum'}], null, '\t');
            // text is '[\n\t"e",\n\t{\n\t\t"pluribus": "unum"\n\t}\n]'

            text = JSON.stringify([new Date()], function (key, value) {
                return this[key] instanceof Date ?
                    'Date(' + this[key] + ')' : value;
            });
            // text is '["Date(---current time---)"]'


        JSON.parse(text, reviver)
            This method parses a JSON text to produce an object or array.
            It can throw a SyntaxError exception.

            The optional reviver parameter is a function that can filter and
            transform the results. It receives each of the keys and values,
            and its return value is used instead of the original value.
            If it returns what it received, then the structure is not modified.
            If it returns undefined then the member is deleted.

            Example:

            // Parse the text. Values that look like ISO date strings will
            // be converted to Date objects.

            myData = JSON.parse(text, function (key, value) {
                var a;
                if (typeof value === 'string') {
                    a =
/^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/.exec(value);
                    if (a) {
                        return new Date(Date.UTC(+a[1], +a[2] - 1, +a[3], +a[4],
                            +a[5], +a[6]));
                    }
                }
                return value;
            });

            myData = JSON.parse('["Date(09/09/2001)"]', function (key, value) {
                var d;
                if (typeof value === 'string' &&
                        value.slice(0, 5) === 'Date(' &&
                        value.slice(-1) === ')') {
                    d = new Date(value.slice(5, -1));
                    if (d) {
                        return d;
                    }
                }
                return value;
            });


    This is a reference implementation. You are free to copy, modify, or
    redistribute.

    This code should be minified before deployment.
    See http://javascript.crockford.com/jsmin.html

    USE YOUR OWN COPY. IT IS EXTREMELY UNWISE TO LOAD CODE FROM SERVERS YOU DO
    NOT CONTROL.
*/

/*jslint evil: true */

/*global JSON */

/*members "", "\b", "\t", "\n", "\f", "\r", "\"", JSON, "\\", call,
    charCodeAt, getUTCDate, getUTCFullYear, getUTCHours, getUTCMinutes,
    getUTCMonth, getUTCSeconds, hasOwnProperty, join, lastIndex, length,
    parse, propertyIsEnumerable, prototype, push, replace, slice, stringify,
    test, toJSON, toString, valueOf
*/

// Create a JSON object only if one does not already exist. We create the
// methods in a closure to avoid creating global variables.

if (!this.JSON) {
    JSON = {};
}
(function () {

    function f(n) {
        // Format integers to have at least two digits.
        return n < 10 ? '0' + n : n;
    }

    if (typeof Date.prototype.toJSON !== 'function') {

        Date.prototype.toJSON = function (key) {

            return this.getUTCFullYear()   + '-' +
                 f(this.getUTCMonth() + 1) + '-' +
                 f(this.getUTCDate())      + 'T' +
                 f(this.getUTCHours())     + ':' +
                 f(this.getUTCMinutes())   + ':' +
                 f(this.getUTCSeconds())   + 'Z';
        };

        String.prototype.toJSON =
        Number.prototype.toJSON =
        Boolean.prototype.toJSON = function (key) {
            return this.valueOf();
        };
    }

    var cx = /[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        escapeable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,
        gap,
        indent,
        meta = {    // table of character substitutions
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '"' : '\\"',
            '\\': '\\\\'
        },
        rep;


    function quote(string) {

// If the string contains no control characters, no quote characters, and no
// backslash characters, then we can safely slap some quotes around it.
// Otherwise we must also replace the offending characters with safe escape
// sequences.

        escapeable.lastIndex = 0;
        return escapeable.test(string) ?
            '"' + string.replace(escapeable, function (a) {
                var c = meta[a];
                if (typeof c === 'string') {
                    return c;
                }
                return '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
            }) + '"' :
            '"' + string + '"';
    }


    function str(key, holder) {

// Produce a string from holder[key].

        var i,          // The loop counter.
            k,          // The member key.
            v,          // The member value.
            length,
            mind = gap,
            partial,
            value = holder[key];

// If the value has a toJSON method, call it to obtain a replacement value.

        if (value && typeof value === 'object' &&
                typeof value.toJSON === 'function') {
            value = value.toJSON(key);
        }

// If we were called with a replacer function, then call the replacer to
// obtain a replacement value.

        if (typeof rep === 'function') {
            value = rep.call(holder, key, value);
        }

// What happens next depends on the value's type.

        switch (typeof value) {
        case 'string':
            return quote(value);

        case 'number':

// JSON numbers must be finite. Encode non-finite numbers as null.

            return isFinite(value) ? String(value) : 'null';

        case 'boolean':
        case 'null':

// If the value is a boolean or null, convert it to a string. Note:
// typeof null does not produce 'null'. The case is included here in
// the remote chance that this gets fixed someday.

            return String(value);

// If the type is 'object', we might be dealing with an object or an array or
// null.

        case 'object':

// Due to a specification blunder in ECMAScript, typeof null is 'object',
// so watch out for that case.

            if (!value) {
                return 'null';
            }

// Make an array to hold the partial results of stringifying this object value.

            gap += indent;
            partial = [];

// If the object has a dontEnum length property, we'll treat it as an array.

            if (typeof value.length === 'number' &&
                    !value.propertyIsEnumerable('length')) {

// The object is an array. Stringify every element. Use null as a placeholder
// for non-JSON values.

                length = value.length;
                for (i = 0; i < length; i += 1) {
                    partial[i] = str(i, value) || 'null';
                }

// Join all of the elements together, separated with commas, and wrap them in
// brackets.

                v = partial.length === 0 ? '[]' :
                    gap ? '[\n' + gap +
                            partial.join(',\n' + gap) + '\n' +
                                mind + ']' :
                          '[' + partial.join(',') + ']';
                gap = mind;
                return v;
            }

// If the replacer is an array, use it to select the members to be stringified.

            if (rep && typeof rep === 'object') {
                length = rep.length;
                for (i = 0; i < length; i += 1) {
                    k = rep[i];
                    if (typeof k === 'string') {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            } else {

// Otherwise, iterate through all of the keys in the object.

                for (k in value) {
                    if (Object.hasOwnProperty.call(value, k)) {
                        v = str(k, value);
                        if (v) {
                            partial.push(quote(k) + (gap ? ': ' : ':') + v);
                        }
                    }
                }
            }

// Join all of the member texts together, separated with commas,
// and wrap them in braces.

            v = partial.length === 0 ? '{}' :
                gap ? '{\n' + gap + partial.join(',\n' + gap) + '\n' +
                        mind + '}' : '{' + partial.join(',') + '}';
            gap = mind;
            return v;
        }
    }

// If the JSON object does not yet have a stringify method, give it one.

    if (typeof JSON.stringify !== 'function') {
        JSON.stringify = function (value, replacer, space) {

// The stringify method takes a value and an optional replacer, and an optional
// space parameter, and returns a JSON text. The replacer can be a function
// that can replace values, or an array of strings that will select the keys.
// A default replacer method can be provided. Use of the space parameter can
// produce text that is more easily readable.

            var i;
            gap = '';
            indent = '';

// If the space parameter is a number, make an indent string containing that
// many spaces.

            if (typeof space === 'number') {
                for (i = 0; i < space; i += 1) {
                    indent += ' ';
                }

// If the space parameter is a string, it will be used as the indent string.

            } else if (typeof space === 'string') {
                indent = space;
            }

// If there is a replacer, it must be a function or an array.
// Otherwise, throw an error.

            rep = replacer;
            if (replacer && typeof replacer !== 'function' &&
                    (typeof replacer !== 'object' ||
                     typeof replacer.length !== 'number')) {
                throw new Error('JSON.stringify');
            }

// Make a fake root object containing our value under the key of ''.
// Return the result of stringifying the value.

            return str('', {'': value});
        };
    }


// If the JSON object does not yet have a parse method, give it one.

    if (typeof JSON.parse !== 'function') {
        JSON.parse = function (text, reviver) {

// The parse method takes a text and an optional reviver function, and returns
// a JavaScript value if the text is a valid JSON text.

            var j;

            function walk(holder, key) {

// The walk method is used to recursively walk the resulting structure so
// that modifications can be made.

                var k, v, value = holder[key];
                if (value && typeof value === 'object') {
                    for (k in value) {
                        if (Object.hasOwnProperty.call(value, k)) {
                            v = walk(value, k);
                            if (v !== undefined) {
                                value[k] = v;
                            } else {
                                delete value[k];
                            }
                        }
                    }
                }
                return reviver.call(holder, key, value);
            }


// Parsing happens in four stages. In the first stage, we replace certain
// Unicode characters with escape sequences. JavaScript handles many characters
// incorrectly, either silently deleting them, or treating them as line endings.

            cx.lastIndex = 0;
            if (cx.test(text)) {
                text = text.replace(cx, function (a) {
                    return '\\u' +
                        ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
                });
            }

// In the second stage, we run the text against regular expressions that look
// for non-JSON patterns. We are especially concerned with '()' and 'new'
// because they can cause invocation, and '=' because it can cause mutation.
// But just to be safe, we want to reject all unexpected forms.

// We split the second stage into 4 regexp operations in order to work around
// crippling inefficiencies in IE's and Safari's regexp engines. First we
// replace the JSON backslash pairs with '@' (a non-JSON character). Second, we
// replace all simple value tokens with ']' characters. Third, we delete all
// open brackets that follow a colon or comma or that begin the text. Finally,
// we look to see that the remaining characters are only whitespace or ']' or
// ',' or ':' or '{' or '}'. If that is so, then the text is safe for eval.

            if (/^[\],:{}\s]*$/.
test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@').
replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']').
replace(/(?:^|:|,)(?:\s*\[)+/g, ''))) {

// In the third stage we use the eval function to compile the text into a
// JavaScript structure. The '{' operator is subject to a syntactic ambiguity
// in JavaScript: it can begin a block or an object literal. We wrap the text
// in parens to eliminate the ambiguity.

                j = eval('(' + text + ')');

// In the optional fourth stage, we recursively walk the new structure, passing
// each name/value pair to a reviver function for possible transformation.

                return typeof reviver === 'function' ?
                    walk({'': j}, '') : j;
            }

// If the text is not JSON parseable, then a SyntaxError is thrown.

            throw new SyntaxError('JSON.parse');
        };
    }
})();

var ParseJSON = JSON.parse;

/////////////////////////////////////////////////////////////////////////////////////////////
///
/// End include
///
/////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////
///
/// @class EarthBrowser
///
/// EarthBrowser control class
///
///////////////////////////////////////////////////////////////////////////
var EarthBrowser = function()
{
	///
	/// Instances of the EarthBrowser control
	var _instances = {};
	var _subframes = {};
	var _base_zindex = 10;
	
	///
	var userAgent = navigator.userAgent.toLowerCase();

	/// Figure out what browser is being used
	var browser = {
		version: (userAgent.match(/.+(?:rv|it|ra|ie)[\/: ]([\d.]+)/) || [])[1],
		safari: /webkit/.test(userAgent),
		opera: /opera/.test(userAgent),
		msie: /msie/.test(userAgent) && !/opera/.test(userAgent),
		mozilla: /mozilla/.test(userAgent) && !/(compatible|webkit)/.test(userAgent),
		chrome: /chrome/.test(userAgent)
	};
	
	///
	var platform = function() {
		var	win = /win/.test(userAgent);
		var	mac = /mac/.test(userAgent);
		return { win:win, mac:mac };
	}();
	
	///
	function create_closure(name, app)	{
		return function(){
			var args = [];
			for(var i=0;i<arguments.length;++i)
				args.push(arguments[i]);
			return app.createObj(name, args);
		}
	}
	
	///
	function map_classes(plugin) {
		var class_list = plugin.getPublicClassList();
		for(var i=0;i<class_list.length; ++i)
			plugin[class_list[i]] = create_closure(class_list[i], plugin);
		return plugin;
	}
	
///////////////////////////////////////////////
///
/// @class earthbrowser
///
/// Our EarthBrowser flash instance controller
///
///////////////////////////////////////////////
	
	///
	function earthbrowser(elem_id, initCB)//, failCB)
	{
		var self = this;
		self.inst_id = elem_id;
		self.listeners = {};
		self.initCB = initCB;
//		self.failCB = failCB;
		self.isOver = false;
		_instances[self.inst_id] = self;
		
		FABridge.addInitializationCallback(elem_id,
			function() {
				self.fref = document.getElementById(self.inst_id);
				self.plugin = map_classes(FABridge[elem_id].root());
				self.wresize();
				self.initCB(self.plugin);
			}
		);
	}
	
	///
	/// public api
	return {
		
		///
		/// Process function routing from Flash to Javascript
		callback: function(inst_id, mname)
		{
			var i, args = [];
			for(i=2;i<arguments.length;++i)
				args.push(arguments[i]);
			if(inst_id != '' && _instances[inst_id]) {
				earthbrowser.prototype[mname].apply(_instances[inst_id], args);
			} else if(inst_id == '') {
				for(inst_id in _instances) {
					earthbrowser.prototype[mname].apply(_instances[inst_id], args);
					break;
				}
			}
		},
		
		/// Insert flash object with inline swfobject.
		create: function(elem_id, initCB, failCB)
		{
			if(swfobject.hasFlashPlayerVersion('9.0.115'))
				return new earthbrowser(elem_id, initCB);//, failCB);
			failCB('Flash player version 9.0.115 or greater required');
			return null;
		}
	};
}();

BIND_NONE				= 0x0000;
BIND_LEFT				= 0x0001;
BIND_RIGHT				= 0x0002;
BIND_TOP				= 0x0010;
BIND_BOTTOM				= 0x0020;
BIND_ALL				= 0x0033;
BIND_FIXED				= 0x0100;

EarthBrowser._moduledefs = {};
EarthBrowser._modules = {};

EarthBrowser.addModule = function(name, moduledef)
{
	EarthBrowser._moduledefs[name] = moduledef;
}

EarthBrowser.loadModule = function(name)
{
	var module = EarthBrowser._modules[name];
	if(!module)
	{
		if(EarthBrowser._moduledefs[name])
			module = EarthBrowser._modules[name] = EarthBrowser._moduledefs[name]();
	}
	return module;
}

function combineObject(added, base)
{
	air.trace(JSON.stringify(added, null, '\t'));	
	air.trace(JSON.stringify(base, null, '\t'));
	for(var i in added)
		base[i] = added[i];
	air.trace(JSON.stringify(base, null, '\t'));
	return base;
}




