/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/


var _countryPopup = null;
var _countries = null;
var _zonePopup = null;
var _zones = null;
var _city_feature = null;
var _cityWind = null;

/**
* CitiesControl
*
* 
* 
*/
function CitiesControl()
{
	
	// set up a load once initialized
//	function userLocReload() {
//		reloadLocations();
//		_plugin.locationManager.removeEventListener("onuserlocations", initialLoad);
//	}
	_plugin.locationManager.addEventListener("onuserlocations", reloadLocations);
	
	
	with(_plugin)
	try
	{
		var icon = new _plugin.Icon({url:c_icon_base+"city.png"});
//		var placemarks = addFeature(new FeatureSet());
		
		var panel = new Pane({binding:BIND_ALL});
		panel.addEventListener("onmousedown", function() {
			flist.clearSelection();
		})
		
		
		// bottom bar container
		var btncont = panel.addPane(new Frame({
			y:-22, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999],// backgroundAlpha:0.5,
			binding:BIND_LEFT|BIND_RIGHT|BIND_BOTTOM
		}));
		
		// add button
		var addBtn = btncont.addPane(new Frame({
			id:'add_location',
			width:24, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999], //backgroundAlpha:0.01,
			helpTip:_language['add_location']
		}));
		addBtn.addPane(new Frame({
			width:addBtn.width, height:addBtn.height,
			frameWidth:1,
			gradientColors:[0xffffff, 0xffffff], gradientAlphas:[0.5, 0.0],
		}));
		addBtn.addPane(new Icon({skin:'smallPlus', x:7, y:6}));
		addBtn.addEventListener("onclick", newLocation);
		
		// gear button
		var gearBtn = btncont.addPane(new Frame({
			id:'edit_location',
			x:24, width:32, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999], //backgroundAlpha:0.01,
			helpTip:_language['edit_location']
		}));
		gearBtn.addPane(new Frame({
			width:gearBtn.width, height:gearBtn.height,
			frameWidth:1,
			gradientColors:[0xffffff, 0xffffff], gradientAlphas:[0.5, 0.0],
		}));
		gearBtn.addPane(new Icon({skin:'gear', x:7, y:6, scaleX:0.75, scaleY:0.75}));
		gearBtn.addPane(new Icon({skin:'downArrow', x:20, y:8, scaleX:0.75, scaleY:0.75}));
		gearBtn.addEventListener("onclick", function() {
		
			var popup = new LocationPopupMenu({x:gearBtn.globalX, y:gearBtn.globalY + gearBtn.height + 1});
			popup.addItem(new EditPopupMenuItem({text:_language['edit_location']}, function() {
				var sitem = flist.selectedItem();
				if(sitem)
					editLocation(sitem.location);
			}));
			popup.addItem(new EditPopupMenuItem({text:_language['del_location']}, function() {
				var sitem = flist.selectedItem();
				if(sitem)
				{
					playSound('snickOff');
					_plugin.locationManager.removeLocation(sitem.location.id);
				}
				reloadLocations();
			}));
		});
		
		// cities list
		flist = new LocationsList(gearBtn, {
			height:-26,
			backgroundAlpha:0.01, frameWidth:0, 
			vscroll:true, binding:BIND_ALL
		});

		panel.addPane(flist.panel);
		
		reloadLocations();
	}	
	catch(err)
	{
		trace(err.toString());
	}
	
	function newLocation() {
	
		if(_cityWind)
		{
			playSound('buzz');
			return;
		}
	
		playSound('snickOn');
		
		// get next id number
		var locs = _plugin.locationManager.userLocations;
		var id = (locs && locs.length > 0) ? locs[locs.length-1].id - 1 : -1;
	
	
		var locData = {
			id:id,
			name:"New Location",
			latitude:_plugin.camera.latitude,
			longitude:_plugin.camera.longitude,
		};
		
		// find closest city for country/admin1/timezone info
		var closeloc = _plugin.locationManager.findClosest(_plugin.camera.currentCoordinate);
		if(closeloc)
		{
			locData.country = closeloc.countryISO;
			locData.admin1 = closeloc.stateID;
			locData.timezone = closeloc.timezone;
		}
	
	//		air.trace(JSON.stringify(locData));
		var nlocation = _plugin.locationManager.addLocation(new _plugin.Location(locData));
		editLocation(nlocation);
	}

	function editLocation(loc)
	{
		if(!_cityWind)
		{
			_cityWind = new LocationWind()
			_cityWind.addEventListener("onclose", reloadLocations);
		}
		_cityWind.setLocation(loc);
	}

	function reloadLocations() {
		flist.clearItems();
		var locations = _plugin.locationManager.userLocations;
		if(locations && locations.length > 0)
		{
			for(var i=0;i<locations.length;++i)
				flist.addItem(new UserLocation(locations[i]));
		}
	}
	
	return {
		id:'my_locations',
		controlIcon:icon,
		controlPanel:panel,
		helpTip:_language['my_locations']
	}
}

// Bootstrap
EarthBrowser.addModule('CitiesControl', CitiesControl);


/**
* LocationWind
*
* Location editing window.
*/
function LocationWind()
{	
	var location = null;
	with(_plugin)
	{		
		var wind = addWindow(new Frame({
			x:200, y:200, width:300, height:180,
			draggable:true,
			frameRadius:10, frameWidth:1,
			gradientColors:[0xcccccc, 0xaaaaaa],// backgroundAlpha:0.5,
		}));
		wind.filters = _dropshadow;
		wind.addEventListener("onclose", function() {
			if(location)
			{
				updateLocationValues();
				locationManager.updateLocation(location);
			}
			
			_countryPopup.closePopup();
			_countryPopup.removeEventListener("onselect", resetStateMenu);
			_zonePopup.closePopup();
			stpop.closePopup();
			
			_cityWind = null;
		});
		
		// close button
		var close = wind.addPane(new Icon({
			skin:'close',
			x:(os=="Mac") ? 4 : wind.width-18, y:4,
			binding:(os=="Mac") ? BIND_TOP|BIND_LEFT : BIND_TOP|BIND_RIGHT
		}));
		close.addEventListener("onclick", function(e) {
			removeWindow(wind);
		});
/*
		// window resizer
		var resizer = wind.addPane(new Resizer({
			x:wind.width-13, y:wind.height-13,
			minWidth:200, minHeight:160,
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		resizer.target = wind;
		*/
		
		// window title
		var title = wind.addPane(new Label({
			x:0, y:2, width:wind.width,
			text:"", size:14, bold:true, align:"center",
			binding:BIND_TOP|BIND_LEFT|BIND_RIGHT|BIND_FIXED
		}));
		title.x = (wind.width - title.width) * 0.5;
		
		
		// Location name
		var cname = wind.addPane(new Input({
			x:10, y:25, width:wind.width - 20, height:18,
			text:"", size:12, bold:false, fixedWidth:true, color:0,
			background:true, border:true, backgroundColor:0xffffff, borderColor:0, 
			binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
		}));
		cname.filters = _smdrop;
		cname.addEventListener("onchange", function() {
			if(location)
				location.name = cname.text;
			title.text = cname.text;
			title.x = (wind.width - title.width) * 0.5;
		});
		cname.selectAll();
		cname.focus(true);
		
		
		function resetStateMenu() {
			stpop.clear();
			
			var cnt = _countries[_countryPopup.value];
			if(cnt)
			{
			 	var states = cnt.getStates();
				if(states.length > 0)
				{
					for(var i=0;i<states.length;++i)
						stpop.addItem(states[i].name);
				}
			}
		}
		
		
		// country 
		wind.addPane(new Label({
			id:'country',
			x:5, y:50,
			text:_language['country'], size:12, bold:true,
		}));
		
		// build the country popup only once
		if(!_countryPopup)
		{
			_countryPopup = new _plugin.Popup({
				x:100, y:50, width:180, height:20,
			});

			var cnts = new Array();
			_countries = locationManager.getCountries();
			for(var i=0;i<_countries.length;++i)
			{
				cnts.push(_countries[i].name);
//				_countryPopup.addItem(_countries[i].name);
			}
			_countryPopup.items = cnts;
		}

		wind.addPane(_countryPopup);
		_countryPopup.addEventListener("onselect", resetStateMenu);
		
		
		// state
		wind.addPane(new Label({
			id:'state',
			x:5, y:75,
			text:_language['state'], size:12, bold:true
		}));
		var stpop = new Popup({
			x:100, y:75, width:180, height:20
		});	
		wind.addPane(stpop);
		resetStateMenu();
		


		// timezone
		wind.addPane(new Label({
			id:'timezone',
			x:5, y:100,
			text:_language['timezone'], size:12, bold:true
		}));

		// build the timezone popup only once
		if(!_zonePopup)
		{
			_zonePopup = new _plugin.Popup({
				x:100, y:100, width:180, height:20,
			});

			var zids = new Array();
			_zones = locationManager.getTimezones();
			for(var i=0;i<_zones.length;++i)
			{
				zids.push(_zones[i].id);
//				_zonePopup.addItem(_zones[i].id);	
			}
			_zonePopup.items = zids;
		}
		wind.addPane(_zonePopup);


		
		// latitude
		wind.addPane(new Label({
			x:5, y:130,
			id:"latitude", text:_language["latitude"], size:12, bold:true,
		}));

		var lat = wind.addPane(new Input({
			x:80, y:130, width:75, height:18,
			text:"",
			maxChars:10, restrict:'\\-0-9.',
			background:true, backgroundColor:0xffffff, 
			border:true, borderColor:0, 
		}));
		lat.filters = _smdrop;
		
		// longitude
		wind.addPane(new Label({
			x:5, y:155,
			id:"longitude", text:_language["longitude"], size:12, bold:true,
		}));
		
		var lon = wind.addPane(new Input({
			x:80, y:155, width:75, height:18,
			text:"",
			maxChars:10, restrict:'\\-0-9.',
			background:true, backgroundColor:0xffffff, 
			border:true, borderColor:0, 
		}));
		lon.filters = _smdrop;
	}
	
	function setLocation(loc)
	{
		location = loc;
		if(location)
		{
			title.text = cname.text = location.name;
			title.x = (wind.width - title.width) * 0.5;
			
			if(location.country)
			{
				_countryPopup.text = location.country.name;
				resetStateMenu();
			}
				
			if(location.state)
				stpop.text = location.state.name;
				
			if(location.timezone)
				_zonePopup.text = location.timezone;
			
			lon.text = Math.floor(location.longitude * 100000)/100000;
			lat.text = Math.floor(location.latitude * 100000)/100000;
		}
	}
	
	function updateLocationValues()
	{
		if(location)
		{
			location.name = cname.text;
			location.countryISO = _plugin.locationManager.getCountryISO(_countryPopup.text);
			location.stateID = _plugin.locationManager.getStateID(location.countryISO, stpop.text);
			location.timezone = _zonePopup.text;
			location.longitude = parseFloat(lon.text);
			location.latitude = parseFloat(lat.text);
		}
	}
	
	return {
		setLocation:setLocation,
		addEventListener:wind.addEventListener,
	};
}


/**
*
*/
function LocationsList(activateBtn, props)
{
	var items = [];
	var selected = null;
	var contents = null;
	
	with(_plugin)
	{
		var panel = new Scroller(props);
		
		contents = new VBox({
			vmargin:1, hmargin:1,
			backgroundAlpha:0,
			binding:BIND_ALL
		});
		panel.setCanvas(contents);
	}
	
	function selectItemEvt(e)
	{
		selected = null;
		for(var i=0;i<items.length;++i)
		{
			if(items[i].panel == e.target)
			{
				selectItem(i);
				break;
			}
		}
	}
	
	function selectItem(index)
	{
		selected = null;
		if(index >= 0 && index < items.length) {
			selected = items[index];
			for(var i=0;i<items.length;++i)
				items[i].select(i == index);
		}
		activateBtn.activate(selected != null);
		return selected;
	}
	
	return {
		panel:panel,
		length:function() { return items.length; },
		selectedItem:function() { return selected; },
		selectItem:selectItem,
		
		addItem:function(item) {
			items.push(item);
			contents.addPane(item.panel);
			item.panel.addEventListener("onclick", selectItemEvt);
		},
		
		clearItems:function() {
			for(var i=0;i<items.length;++i)
				items[i].panel.removeEventListener("onclick", selectItemEvt);
			items = [];
			selected = null;
			activateBtn.activate(false);
			contents.removeAllPanes();
		},
		
		clearSelection:function() {
			for(var i=0;i<items.length;++i)
				items[i].select(false);
			activateBtn.activate(false);
//		},
		
//		itemIndex:function(item) {
//		},
		
//		editSelected:function() {
//			if(selected)
//				selected.edit();
		}
	};
}




/**
* UserLocation
*
* List element for the search result list. Clicking will zoom camera to location.
*/
function UserLocation(loc)
{
	var selected = false;
	
	with(_plugin)
	{
		var cnt = loc.country;
		var state = loc.state;
		var st_name = state ? state.name : '';
		var cname = loc.name + (st_name.length > 0 ? (', ' + st_name) : '') + ', ' + (cnt ? cnt.name : '');
		
		var panel = new Frame({
			width:1000, height:24,
			backgroundColor:0xffffff, backgroundAlpha:0.5,
		});
		panel.addPane(new Icon({x:4, y:1, url:c_icon_base+"city.png"}));
		panel.addPane(new Label({x:28, y:2, text:cname}));
		panel.addEventListener('onclick', function(e) {
			camera.properties = {longitude:loc.longitude, latitude:loc.latitude};
			_weather_wind.setLocation(loc);
		});
	}
	
	return {
		panel:panel,
		location:loc,
//		itemID:function() { return itemdef.id; },
		selected:function() { return selected; },
		select:function(on) {
			selected = on;
			panel.backgroundColor = selected ? _selColor : 0xffffff;
//			if(selected)
//				setFavoriteView();
		},
		edit:function() {
//			item.in_place_edit();
		}
	};
}



/**
*
*/
function LocationPopupMenu(props)
{
//	x:button.globalX, y:button.globalY + button.height + 1
		
	with(_plugin)
	{
		props.vmargin = 4;
//		props.hmargin = 4;
		props.width = 100;
		props.height = 40;
		props.frameWidth = 1;
		props.frameRadius = 6;
		props.gradientColors = [0xeeeeee, 0xbbbbbb];
		props.backgroundAlpha = 1;
		
		var popup = addPane(new VBox(props));

		function closePopup() {
			_plugin.removePane(popup);
			_plugin.removeEventListener("onmouseup", closePopup, true);
			popup = null;
		}
		
		function addItem(item) {
			item.setMenu(this);
			popup.addPane(item.panel);
		}
		
		_plugin.addEventListener("onmouseup", closePopup, true);
	}
	
	return {
		panel:popup,
		addItem:addItem,
		closePopup:closePopup
	};
}

/**
*
*/
function LocationPopupMenuItem(props, func)
{
	var menu = null;
	var panel = new _plugin.Frame({
		width:0, height:16,
		backgroundColor:0xffffff, backgroundAlpha:0.25,
		binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
	});
	panel.addPane(new _plugin.Label(props));
		
	panel.addEventListener('onclick', function(e) {
		menu.closePopup();
		func(e);
	});
	
	return {
		panel:panel,
		setMenu:function(m) {
			menu = m;
		}
	};
}
