/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/

/**
* ControlWind
* 
*/
function ControlWind()
{
	var stateID = "_control_window";
	var c_tbar_height = 18;
	var curModuleID = '';
	var modules = {}, tabs = {};
	var gridHeight = 180, gridWidth = 165;
	var minHeight = 220, minWidth = 280;
	var activeGrad = [0x777777, 0xaaaaaa], inactiveGrad = [0xeeeeee, 0xaaaaaa];
	var state = _cache.load(stateID, {
		module:'datasets',
		listViewState:false,
		shrunk:false,
		x:0, y:0,
		width:280, height:380
	});
	
	function saveState() {
		state.x = wind.x;
		state.y = wind.y;
		if(!state.shrunk && state.listViewState)
		{
			state.width = Math.max(windBack.width_, minWidth);
			state.height = Math.max(windBack.height_, minHeight);
		}
		state.module = curModuleID;
	}
	
	
	function toggleShrinkWindow() {
		if(!state.shrunk)
			contents.visible = resizer.visible = false;	
			
		var wheight = state.listViewState ? state.height : c_tbar_height + 160;
			
		windBack.tweenProperties(0.4, {height:state.shrunk ? wheight : c_tbar_height}, function() {
			state.shrunk = !state.shrunk;
			if(state.shrunk)
			{
				contents.visible = resizer.visible = false;
			}
			else
			{
				contents.alpha = 0;
				contents.visible = resizer.visible = true;
				contents.tweenProperties(0.4, {autoAlpha:1});
			}
			
//			contents.visible = resizer.visible = !state.shrunk;
			saveState();
		});
	}
	
	function toggleGridView() {
		playSound('snickOn');
		state.listViewState = !state.listViewState;
		
		displayMode.properties = {
			id:state.listViewState ? 'grid_view' : 'full_list_view',
			skin:state.listViewState ? "blocks" : "bars",
			helpTip:state.listViewState ? _language['grid_view'] : _language['full_list_view']
		}
		
		var wheight = state.listViewState ? windBack.height_-120 : windBack.height_ - 90;
//		zoomControl.dockControl.properties = {
//			y:state.listViewState ? 50 : c_tbar_height + 4,
//			height:wheight
//		};
		
		timeControl.setGridView(!state.listViewState);
		
		resizer.visible = state.listViewState;
		gridViewContainer.visible = listViewContainer.visible = false;
			
		windBack.tweenProperties(0.5, {
				width:state.listViewState ? state.width : 165,
				height:state.listViewState ? state.height : c_tbar_height + 160
			},
			function() {
				
				if(state.listViewState)
				{
					listViewContainer.alpha = 0;
					listViewContainer.visible = resizer.visible = true;
					listViewContainer.tweenProperties(0.4, {autoAlpha:1});
				}
				else
				{
					gridViewContainer.alpha = 0;
					gridViewContainer.visible = true;
					gridViewContainer.tweenProperties(0.4, {autoAlpha:1});
				}
				
//				zoomControl.resetSlider();
				
				if(state.shrunk)
					toggleShrinkWindow();
			}
		);
		
	}
	
	function tryFadeOut() {
		wind.tweenProperties(8.0, {}, function(){
			wind.tweenProperties(1.0, {alpha:0.4});
		});
	}
	function tryFadeIn() {
		wind.tweenProperties(0.1, {alpha:1.0});
	}
	
	with(_plugin)
	{
		var wwidth = state.listViewState ? state.width : gridWidth;
		var wheight = state.listViewState ? state.height : gridHeight;
	
		var wind = new Pane({
			x:state.x, y:state.y, width:wwidth, height:wheight,
			draggable:true, bindEdge:true
		});
		wind.filters = _dropshadow;
		wind.addEventListener("ondragend", saveState);
		
		var windBack = wind.addPane(new Frame({
			width:wwidth, height:wheight,
			frameRadius:12, frameWidth:1,
			gradientColors:[0x999999, 0xeeeeee], backgroundAlpha:0.65,
			binding:BIND_ALL
		}));
		
		
		// window resizer
		var resizer = windBack.addPane(new Resizer({
			x:wwidth-13, y:wheight-13,
			visible:state.listViewState,
			minWidth:minWidth, minHeight:minHeight,
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		resizer.target = windBack;
		resizer.addEventListener("onresizeend", saveState);
		
		
		// main contents
		var contents = windBack.addPane(new Pane({
			width:wwidth, height:wheight,
			binding:BIND_ALL
		}));
		
		
		
		// titlebar
		var tbar = windBack.addPane(new Frame({
			x:1, y:0, width:wwidth-1, height:c_tbar_height,
			frameRadius:10,// frameWidth:1, frameAlpha:0.3,
			gradientColors:[0x999999, 0x666666],
//			gradientColors:[0xE52E2A, 0xC50E0A],
			backgroundAlpha:0.6,
			binding:BIND_LEFT|BIND_RIGHT|BIND_TOP
		}));
		tbar.addEventListener("ondoubleclick", toggleShrinkWindow);
		var thighlight = tbar.addPane(new Frame({
			x:2, y:1, width:wwidth-4, height:12,
			frameRadius:6,
			gradientColors:[0xffffff, 0xffffff],
			gradientAlphas:[0.5, 0.0],
			binding:BIND_LEFT|BIND_RIGHT|BIND_TOP
		}));
		thighlight.addEventListener("ondoubleclick", toggleShrinkWindow);
		
		// shrink
		var shrink = tbar.addPane(new Frame({
			x:wwidth-28, y:3, width:20, height:10,
			frameRadius:11, frameWidth:1, frameColor:0x222222, frameAlpha:0.5,
			gradientColors:[0xffffff, 0x888888],
			binding:BIND_TOP|BIND_RIGHT
		}));
		shrink.filters = [new DropShadowFilter(1, 90, 0 ,0.5)];
		shrink.addEventListener("onmousedown", function() { 
			playSound(state.shrunk ? 'snickOff' : 'snickOn');
			toggleShrinkWindow();
		});
		shrink.addEventListener("onmouseover", function() { shrink.gradientColors = [0xcccccc, 0x666666]; });
		shrink.addEventListener("onmouseout", function() { shrink.gradientColors = [0xffffff, 0x888888]; });
		
		
		// display mode
		var displayMode = tbar.addPane(new Icon({
			x:10, y:4,
			id:state.listViewState ? 'grid_view' : 'full_list_view',
			skin:state.listViewState ? "blocks" : "bars",
			helpTip:state.listViewState ? _language['grid_view'] : _language['full_list_view']
		}));
		displayMode.filters = [new DropShadowFilter(1, 90, 0 ,0.5)];
		displayMode.addEventListener("onclick", toggleGridView);
		
		
		// list view container
		var listViewContainer = contents.addPane(new Pane({
			y:c_tbar_height + 2, width:wwidth, height:wheight,
			visible:state.listViewState,
			binding:BIND_ALL
		}));
		
		// grid view container
		var gridViewContainer = contents.addPane(new Pane({
			x:1, y:c_tbar_height+1, width:wwidth, height:wheight,
			visible:!state.listViewState,
			binding:BIND_ALL
		}));
		
		// module icon container
		var iconContainer = listViewContainer.addPane(new HBox({
			x:5, y:0, hpad:1, backgroundAlpha:0
		}));
		
		// module pane container
		var container = listViewContainer.addPane(new Frame({
			x:5, y:26, width:wwidth-30, height:wheight-90,
			frameWidth:1,
			backgroundAlpha:0.75,
			binding:BIND_ALL
		}));
		
		/*
		var cpicker = contents.addPane(new ColorSelector({
			x:2, y:20, width:128, height:128
		}));
		cpicker.addEventListener("onupdate", function(){
			tbar.gradientColors = [cpicker.color, cpicker.color];
		});
		
		var cslider = contents.addPane(new Slider({
			x:200, y:20,
			invert:true, vertical:true,
			binding:BIND_TOP|BIND_BOTTOM|BIND_LEFT
		}));
		cslider.setBack(new VBar({ skin:"zoom_bar", height:20}));
		cslider.setThumb(new Icon({ skin:"zoom_thumb" }));
		cslider.addEventListener("onslider", function(e) { cpicker.brightness = cslider.value*255; });
		
		*/
		
		// mouse over fades and position storage
		wind.addEventListener("onmouseover", tryFadeIn);
		wind.addEventListener("onmouseout", tryFadeOut);
		
		
		// add control panes to the top of the list
		var timeControl = new EarthBrowser.loadModule('TimeControl');
		var ctl = contents.addPane(timeControl.dockControl, 0);
		ctl.properties = {x:5, y:wind.height_-36, width:wind.width_ - 28, binding:BIND_BOTTOM|BIND_LEFT|BIND_RIGHT};
//		ctl.properties = {x:5, y:wheight-36, width:wwidth - 20, binding:BIND_BOTTOM|BIND_LEFT|BIND_RIGHT};
		timeControl.setGridView(!state.listViewState);
		
		var zoomControl = new EarthBrowser.loadModule('ZoomControl');
		ctl = contents.addPane(zoomControl.dockControl, 0);
		ctl.properties = {
			x:wwidth-17, y:c_tbar_height + 4,
			height:windBack.height_-60,
//			x:wwidth-17, y:state.listViewState ? 50 : c_tbar_height + 4,
//			height:state.listViewState ? windBack.height_-120 : windBack.height_ - 90,
			binding:BIND_TOP|BIND_BOTTOM|BIND_RIGHT
		};
		
		if(state.shrunk)
		{
			contents.visible = resizer.visible = false;
			windBack.resizeTo(windBack.width_, c_tbar_height);
		}
		
		tryFadeOut();
	}
	
	function getModule() {
		return modules[curModuleID];
	}
	
	function setModule(mid, changeState) {
		
		if(changeState && !state.listViewState)
			toggleGridView();
			
		for(var tid in tabs)
			tabs[tid].gradientColors = (tid == mid) ? activeGrad : inactiveGrad;
		
		var module = modules[mid];
		if(module)
		{
			var cmod = modules[curModuleID];
			if(cmod)
			{
				if(cmod.controlPanel)
					container.removePane(cmod.controlPanel);
				if(cmod.deactivate)
					cmod.deactivate();
			}

			curModuleID = state.module = module.id;

			if(module.controlPanel)
			{
				container.addPane(module.controlPanel);
				module.controlPanel.properties = {x:0, y:0, width:container.width_, height:container.height_};
			}
			
			if(module.activate)
				module.activate();
		}
		return module;
	}
	
	function addPanelModule(module) {
		if(!module.id)
			return;
			
		var btnwidth = 30;
		modules[module.id] = module;
		var tab = tabs[module.id] = iconContainer.addPane(new _plugin.Frame({
			width:btnwidth, height:38, frameRadius:10, frameWidth:1,
			id:module.id ? module.id : null,
			helpTip:module.helpTip ? module.helpTip : null,
			gradientColors:inactiveGrad, backgroundAlpha:0.6,
			scrollHeight:26
		}));
		tab.addEventListener("onclick", function() {
			playSound('snickOn');
			setModule(module.id, false);
		});
		
//		module.controlIcon.properties = {x:7, y:1};
		var ico = tab.addPane(module.controlIcon);
		ico.addEventListener("onload", function() {
			ico.properties = {x:(btnwidth - ico.width)/2, y:(32 - ico.height)/2- 2}
		});
		
		if(module.id == state.module)
			setModule(state.module, false);
		
		if(module.gridPanel)
		{
			gridViewContainer.addPane(module.gridPanel);
			module.gridPanel.properties = {width:container.width_, height:container.height_};
		}
	}
	return {
		wind:wind,
		addModule:addPanelModule,
		setModule:setModule,
		getModule:getModule,
		curModuleID:function() { return curModuleID; }
	};
}

// Bootstrap
EarthBrowser.addModule('ControlWindow', ControlWind);