/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/

var _weather_wind = null;


/**
* DatasetsCtl
*
*/
function DatasetsCtl()
{
	var gridPanel = null;
	var features = null;
	var gridSpinner = null;
	var buttons = {};
	var durl = "http://api.earthbrowser.com/base_dataset_v32/";
	var dsetlist = [
		'__weather__', '__clouds__', '_doppler', '_quakes',
		'_storms', '_satellite', '_buoys', '_wildfires', '_nws_warning',
		'_volcanoes', '_tec_plates', '_countries', '_world_borders', '_us_borders',
		'_webcams', '_panaramio', '_aurora'
	];
	var dgrouplist = [
		'_us_fcst', '_global_fcst'
	];
	
	var grid_order = [
		'__clouds__', '_doppler', '_global_fcst', '_us_fcst', '__weather__', 
		'_tec_plates',  '_volcanoes', '_quakes', '_nws_warning','_storms', 
		'_panaramio',  '_webcams', '_buoys', '_wildfires',  '_satellite',
		'_settings', '_world_borders', '_us_borders', '_countries', '_aurora', 
	];
	
	function buildGridPanel()
	{
		features.removeEventListener("onload", buildGridPanel);
		
		gridSpinner.stop();
		gridPanel.removePane(gridSpinner);
		gridSpinner = null;
		
		
//		buttons['night'] = new OptionButton({id:"night_shadows", vid:"night", url:c_icon_base+"night_icon.png"});
//		buttons['grid'] = new OptionButton({id:"grid", vid:"grid", url:c_icon_base+"grid_icon.png"});
		buttons['_settings'] = new SettingsPopupButton();
		buttons['_satellite'] = new SatelliteButton();
		
		for(var i=0;i<dsetlist.length;++i)
		{
			var dset = features.findFeature(dsetlist[i]);
			if(dset)
			{
				// xxxxx FIX THIS by removing _satellite dataset from v3.2 kml xxxxx
				if(dset.id != '_satellite') {
//				air.trace('found: ' + dsetlist[i]);
				var btn = new DSButton(dset);
				buttons[btn.id] = btn;
				}
			}
		}
		
		for(var i=0;i<dgrouplist.length;++i)
		{
			var dset = features.findFeature(dgrouplist[i]);
			if(dset)
			{
				var btn = new DSPopupButton(dset);
				buttons[btn.id] = btn;
			}
		}
		
		for(var i=0;i<grid_order.length;++i)
		{
			var id = grid_order[i];
			if(buttons[id])
			{
				var ico = gridPanel.addPane(buttons[id].panel);
				ico.properties = {x:(i % 5) * 28 + 2, y:Math.floor(i / 5) * 28 + 1};	
			}
		}
		
		
		// special statellite functionality
		var satds = features.findFeature('_satellite')
		if(satds)
		{
			satds.visibility = false;
			/*
			// listen for a target change
			var lastTarget = null;
			_plugin.camera.addEventListener("ontargetstatus", function(e) {
				if(_plugin.camera.targetFeature == null)
					lastTarget = null;
			});
			
			satds.addEventListener("onclick", function(e) {
				e.stopPropagation();
				
				if(lastTarget && lastTarget.id == e.target.id)
				{
					_plugin.addWindow(new _plugin.InfoWindow({
						x:200+_iwindoff, y:60+_iwindoff, width:400, height:300,
						text:e.target.description, addressForce:true
					}));
					_iwindoff += 10;
				}
				lastTarget = e.target;
				
				_plugin.camera.properties = {altitude:8e6, speed:0.005};
				_plugin.camera.targetFeature = e.target;
			});
			*/
		}
	}
	
	
	with(_plugin)
	{
		var icon = new Icon({url:c_icon_base+"datasets_icon.png"});

		// create weather window
		_weather_wind = new WeatherWind();
		addWindow(_weather_wind.wind);
		locationManager.addEventListener("onclick", function(e) {
			_weather_wind.setLocation(e.target);
		});
	
		gridPanel = new Pane({binding:BIND_ALL});
		gridSpinner = gridPanel.addPane(new Spinner({x:16, y:16}));
		gridSpinner.start();

		// load the Dataset features
		features = addFeature(new KMLFile({name:_language['datasets'], open:true, url:durl}));
		features.addEventListener("onload", buildGridPanel);
		features.editable = false;
	
		// listen for weather icon clicks
		features.addEventListener("onweather", function(e) {
//			_plugin.trace("onweather: " + e.target);
			_weather_wind.setLocationIcon(e.target);
		});
	
		var panel = new Pane({binding:BIND_ALL});
	
		var scrl = panel.addPane(new Scroller({
			x:3, y:1,
			vscroll:true,
			backgroundAlpha:0.01, frameWidth:0,
			binding:BIND_ALL
		}));
		scrl.setCanvas(features.listView);
		
		
		// check for language changes
		_plugin.addEventListener("onlanguagechange", function() {
			features.name = _language['datasets']
			for(var i in buttons)
			{
				try { buttons[i].resetLang(); }
				catch(err) { air.trace('langerr: ' + err.toString()); }
			}
		});
	}
	
	return {
		id:'datasets',
		controlIcon:icon,
		controlPanel:panel,
		gridPanel:gridPanel,
		helpTip:_language['datasets']
	}
}

// Bootstrap
EarthBrowser.addModule('DatasetsControl', DatasetsCtl);


var dsActiveGrad = [0x777777, 0xaaaaaa], dsInactiveGrad = [0xeeeeee, 0xaaaaaa];

var sidx = 0;
function DSButton(dataset)
{
	with(_plugin)
	{
		var panel = new Frame({
			width:28, height:28,
			frameRadius:6, frameWidth:1, frameAlpha:0.5,
			gradientColors:dataset.visibility ? dsActiveGrad : dsInactiveGrad,
			helpTip:dataset.name
		});
		panel.filters = _smdrop;
		panel.addEventListener("onclick", function() {
			playSound(dataset.visibility ? 'snickOff' : 'snickOn');
			dataset.visibility = !dataset.visibility;
		});
//		panel.addEventListener("onmouseover", function() { _mover.play(); });
		
		var ico = panel.addPane(new Icon({
			x:3, y:3,
			href:dataset.listIconUrl
		}));
		ico.addEventListener("onload", function() {
			ico.properties = {x:(28 - ico.width)/2, y:(28 - ico.height)/2}
		});
			
		dataset.addEventListener("onupdate", function() {
			panel.gradientColors = dataset.visibility ? dsActiveGrad : dsInactiveGrad;
		});
	}
		
	return {
		id:dataset.id,
		panel:panel,
		resetLang:function() { panel.helpTip = dataset.name }
	};
}

function DSPopupButton(dataset)
{
	var popup = null;

	function closeFunc() {
		popup = null;
	}

	with(_plugin)
	{
		var panelContainer = new Pane({width:28, height:28});
		
		var panel = panelContainer.addPane(new Frame({
			width:28, height:28,
			frameRadius:6, frameWidth:1, frameAlpha:0.5,
			gradientColors:dsInactiveGrad,
			helpTip:dataset.name
		}));
		panel.filters = _smdrop;
		panel.addEventListener("onclick", function() {
			if(!popup)
			{
				playSound('snickOn');
				popup = new DatasetPopup(dataset, panelContainer, closeFunc);
			}
		});
	
		var ico = panel.addPane(new Icon({
			x:3, y:3,
			href:dataset.listIconUrl
		}));
		ico.addEventListener("onload", function() {
			ico.properties = {x:(28 - ico.width)/2, y:(28 - ico.height)/2}
		});
		
		var arrow = panel.addPane(new Icon({x:15, y:15, skin:"wind_reduce"}));
		arrow.filters = _dropshadow;
	}
		
	return {
		id:dataset.id,
		panel:panelContainer,
		popupClosed:function() { popup = null; },
		resetLang:function() { panel.helpTip = dataset.name }
	};
}



function DatasetPopup(dataset, btnPanel, closeFunc)
{
	function closeDSPopup()
	{
		closeFunc();
		close.removeEventListener("onclick", closeDSPopup);
		btnPanel.parent.removePane(panel);
	}
	
	with(_plugin)
	{
		var panel = new VBox({
			hmargin:4, vmargin:2, vpad:6,
			width:160, height:dataset.numChildren * 30,
			frameRadius:6, frameWidth:1, frameAlpha:0.5,
			gradientColors:dsInactiveGrad,
//			gradientColors:[0x0000dd, 0x0000aa],
			backgroundAlpha:0.75
		});
		
		var close = panel.addPane(new Icon({skin:"close", x:panel.width-14, y:2}));
		close.addEventListener("onclick", closeDSPopup);
		
		var chld = dataset.getChildren();
		for(var i=0;i<chld.length;++i)
			panel.addPane(new DatasetPopupItem(chld[i]).panel);
		
		panel.fixSize();
		
		panel.properties = {x:btnPanel.x + 4, y:btnPanel.y + btnPanel.height/2};
		btnPanel.parent.addPane(panel);
	}
	
	return {
		panel:panel
	};
}

///
/// DatasetPopupItem
///
function DatasetPopupItem(dsitem)
{
	var panel = new _plugin.Checkbox({ text:dsitem.name, value:dsitem.visibility });
	panel.addEventListener("onupdate", function() { dsitem.visibility = panel.value; });
	var ico = panel.addPane(new _plugin.Icon({ x:3, y:3, href:dsitem.listIconUrl }), 1);
	return {
		panel:panel
	};
}

///
/// SatelliteButton
///
function SatelliteButton()
{
	with(_plugin)
	{
		var panelContainer = new Pane({width:28, height:28});
		
		var panel = panelContainer.addPane(new Frame({
			width:28, height:28,
			frameRadius:6, frameWidth:1, frameAlpha:0.5,
			gradientColors:_satFeatureset.visibility ? dsActiveGrad : dsInactiveGrad,
			helpTip:_language['settings']
		}));
		panel.filters = _smdrop;
		panel.addEventListener("onclick", function() {
			playSound('snickOn');
			_satFeatureset.visibility = !_satFeatureset.visibility;
			_cache.store("_satellitesOn", _satFeatureset.visibility);
		});
	
		var ico = panel.addPane(new Icon({
			x:3, y:3, href:c_icon_base+"swift.png"
		}));
		ico.addEventListener("onload", function() {
			ico.properties = {x:(28 - ico.width)/2, y:(28 - ico.height)/2}
		});
		
		_satFeatureset.addEventListener("onupdate", function() {
			panel.gradientColors = _satFeatureset.visibility ? dsActiveGrad : dsInactiveGrad;
		});
	}
	
	return {
		id:'_satellites',
		panel:panelContainer,
		resetLang:function() { panel.helpTip = _language['eb_satellite'] }
	};
}




///
/// SettingsPopupButton
///
function SettingsPopupButton()
{
	var popup = null;
	
	function closeFunc() {
		popup = null;
	}
	
	with(_plugin)
	{
		var panelContainer = new Pane({width:28, height:28});
		
		var panel = panelContainer.addPane(new Frame({
			width:28, height:28,
			frameRadius:6, frameWidth:1, frameAlpha:0.5,
			gradientColors:dsInactiveGrad,
			helpTip:_language['settings']
		}));
		panel.filters = _smdrop;
		panel.addEventListener("onclick", function() {
			if(!popup)
			{
				playSound('snickOn');
				popup = new SettingsPopup(panelContainer, closeFunc);	
			}
		});
//		panel.addEventListener("onmouseover", function() { _mover.play(); });
	
		var ico = panel.addPane(new Icon({
			x:3, y:3, href:c_icon_base+"settings_icon.png"
		}));
		ico.addEventListener("onload", function() {
			ico.properties = {x:(28 - ico.width)/2, y:(28 - ico.height)/2}
		});
		
		panel.addPane(new Icon({x:15, y:15, skin:"wind_reduce"}));
	}
	
	return {
		id:'_settings',
		panel:panelContainer,
		popupClosed:function() { popup = null; },
		resetLang:function() { panel.helpTip = _language['settings'] }
	};
}

///
/// SettingsPopup
///
function SettingsPopup(btnPanel, closeFunc)
{
	var settings_list = [
		{id:"night_shadows", vid:"night", url:c_icon_base+"night_icon.png"},
		{id:"city_lights", vid:"lights", url:c_icon_base+"lights_icon.png"},
		{id:"stars", vid:"stars", url:c_icon_base+"stars_icon.png"},
		{id:"grid", vid:"grid", url:c_icon_base+"grid_icon.png"},
		{id:"show_time", vid:"showTime", url:c_icon_base+"show_time_icon.png"},
		{id:"time_24hour", vid:"time24hr", url:c_icon_base+"24hour_icon.png"},
		{id:"metric", vid:"metric", url:c_icon_base+"ruler.png"},
		{id:"help_tips", vid:"helpTips", url:c_icon_base+"help_balloon.png"},
	];
	
	function closePopup()
	{
		closeFunc();
		close.removeEventListener("onclick", closePopup);
		btnPanel.parent.removePane(panel);
	}

	with(_plugin)
	{
		var panel = new VBox({
			hmargin:4, vmargin:2, vpad:6,
			width:160, height:30,
			frameRadius:6, frameWidth:1, frameAlpha:0.5,
			gradientColors:dsInactiveGrad,
//			gradientColors:[0x0000dd, 0x0000aa],
			backgroundAlpha:0.75
		});
		
		var close = panel.addPane(new Icon({skin:"close", x:panel.width-14, y:2}));
		close.addEventListener("onclick", closePopup);
		
		for(var i=0;i<settings_list.length;++i)
			panel.addPane(new SettingsPopupItem(settings_list[i]).panel);
		panel.addPane(new ShowCoordsPopupItem().panel);
		
		panel.fixSize();
		
		panel.properties = {x:btnPanel.x + 4, y:btnPanel.y + btnPanel.height/2};
		btnPanel.parent.addPane(panel);
	}
	
	return {
		panel:panel
	};
}

///
/// SettingsPopupItem
///
function SettingsPopupItem(itemdef)
{
	with(_plugin)
	{
		var panel = new Checkbox({id:itemdef.id, x:28, y:2, text:_language[itemdef.id], value:viewOptions[itemdef.vid]});
		var ico = panel.addPane(new Icon({url:itemdef.url}), 1);
		ico.addEventListener("onload", function() {
			ico.properties = {x:(28 - ico.width)/2, y:(28 - ico.height)/2}
		});
		
		panel.addEventListener("onupdate", function() { viewOptions[itemdef.vid] = panel.value; });
		viewOptions.addEventListener("onupdate", function() { panel.value = viewOptions[itemdef.vid]; });
	}
	
	return {
		panel:panel
	};
}


///
/// ShowCoordsPopupItem
///
function ShowCoordsPopupItem()
{
	with(_plugin)
	{
		var panel = new Checkbox({id:'showCoords', x:28, y:2, text:_language['showCoords'], value:_coords.getVisible()});
		var ico = panel.addPane(new Icon({url:c_icon_base+"crosshair.png"}), 1);
		ico.addEventListener("onload", function() {
			ico.properties = {x:(28 - ico.width)/2, y:(28 - ico.height)/2}
		});

		panel.addEventListener("onupdate", function() {
			_coords.setVisible(panel.value);
		});
	}

	return {
		panel:panel
	};
}
