/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/


var _mapLayers = null;

/**
* Favorites
*/
function FavoritesView()
{
	var flist = null;
	var selected = null;
	var windGrad = [0x888888, 0xdddddd], btnGrad = [0xffffff, 0xaaaaaa];
	
	_mapLayers = EarthBrowser.loadModule('MapLayers');
	
	function newFavorite() {
		playSound('snickOn');
		
		var data = JSON.stringify({
			latitude:_plugin.camera.latitude,
			longitude:_plugin.camera.longitude,
			altitude:_plugin.camera.altitude,
			mapLayer:_mapLayers.getLayer(),
			viewOptions:_plugin.viewOptions.properties
		});
//		air.trace(data);
		_plugin.database.execute("insert into favorites (name, data) VALUES(:name, :data)", { name:"New Favorite", data:data } );
		reloadItems();
		var item = flist.selectItem(flist.length()-1);
		if(item)
			item.edit();
	}


	function reloadItems() {
		flist.clearItems();
		var defs = _plugin.database.execute("select id, name, data from favorites");
		if(defs)
		{
			for(var i=0;i<defs.length;++i)
				flist.addItem(new FavoriteItem(defs[i]));
		}
	}

	
	with(_plugin)
	{
		// make sure we have a table
//		database.execute("drop table favorites");
		database.execute("create table if not exists favorites (id integer primary key, name text, data text)");
		
		var icon = new Icon({url:c_icon_base+"favorites.png"});

		var panel = new Pane({
			binding:BIND_ALL
		});
		panel.addEventListener("onmousedown", function() {
			flist.clearSelection();
		})
		
		var btncont = panel.addPane(new Frame({
			y:-22, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999],// backgroundAlpha:0.5,
			binding:BIND_LEFT|BIND_RIGHT|BIND_BOTTOM
		}));
		
		
		// add button
		var addBtn = btncont.addPane(new Frame({
			id:'create_favorite',
			width:24, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999], //backgroundAlpha:0.01,
			helpTip:_language['create_favorite']
		}));
		addBtn.addPane(new Frame({
			width:addBtn.width, height:addBtn.height,
			frameWidth:1,
			gradientColors:[0xffffff, 0xffffff], gradientAlphas:[0.5, 0.0],
			helpTip:_language['create_favorite']
		}));
		addBtn.addPane(new Icon({skin:'smallPlus', x:7, y:6}))
		addBtn.addEventListener("onclick", newFavorite);
		
		// gear button
		var gearBtn = btncont.addPane(new Frame({
			id:'edit_favorite',
			x:24, width:32, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999], //backgroundAlpha:0.01,
			helpTip:_language['edit_favorite']
		}));
		gearBtn.addPane(new Frame({
			width:gearBtn.width, height:gearBtn.height,
			frameWidth:1,
			gradientColors:[0xffffff, 0xffffff], gradientAlphas:[0.5, 0.0],
		}));
		gearBtn.addPane(new Icon({skin:'gear', x:7, y:6, scaleX:0.75, scaleY:0.75}));
		gearBtn.addPane(new Icon({skin:'downArrow', x:20, y:8, scaleX:0.75, scaleY:0.75}));
		gearBtn.addEventListener("onclick", function() {
			var popup = new EditPopupMenu({x:gearBtn.globalX, y:gearBtn.globalY + gearBtn.height + 1});
			popup.addItem(new EditPopupMenuItem({text:_language['rename']}, function() {
				flist.editSelected();
			}));
			popup.addItem(new EditPopupMenuItem({text:_language['del']}, function() {
				_plugin.database.execute("delete from favorites where id=:id", {id:flist.selectedItem().itemID()})
				reloadItems();
			}));
		});
		gearBtn.activate(false);
		
				
		// favorites list
		flist = new FavoritesList(gearBtn, {
			height:-26,
			backgroundAlpha:0.01, frameWidth:0, 
			vscroll:true, binding:BIND_ALL
		});
		
		panel.addPane(flist.panel);
		
		reloadItems();
	}
	
	return {
		id:'favorite_views',
		controlIcon:icon,
		controlPanel:panel,
		helpTip:_language['favorite_views']
	}
}


// Bootstrap
EarthBrowser.addModule('Favorites', FavoritesView);

/**
*
*/
function EditPopupMenu(props)
{
//	x:button.globalX, y:button.globalY + button.height + 1
		
	with(_plugin)
	{
		props.vmargin = 4;
//		props.hmargin = 4;
		props.width = 100;
		props.height = 40;
		props.frameWidth = 1;
		props.frameRadius = 6;
		props.gradientColors = [0xeeeeee, 0xbbbbbb];
		props.backgroundAlpha = 1;
		
		var popup = addPane(new VBox(props));

		function closePopup() {
			_plugin.removePane(popup);
			_plugin.removeEventListener("onmouseup", closePopup, true);
			popup = null;
		}
		
		function addItem(item) {
			item.setMenu(this);
			popup.addPane(item.panel);
		}
		
		_plugin.addEventListener("onmouseup", closePopup, true);
	}
	
	return {
		panel:popup,
		addItem:addItem,
		closePopup:closePopup
	};
}

/**
*
*/
function EditPopupMenuItem(props, func)
{
	var menu = null;
	var panel = new _plugin.Frame({
		width:0, height:16,
		backgroundColor:0xffffff, backgroundAlpha:0.25,
		binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
	});
	panel.addPane(new _plugin.Label(props));
		
	panel.addEventListener('onclick', function(e) {
		menu.closePopup();
		func(e);
	});
	
	return {
		panel:panel,
		setMenu:function(m) {
			menu = m;
		}
	};
}


/**
*
*/
function FavoritesList(activateBtn, props)
{
	var items = [];
	var selected = null;
	var contents = null;
	
	with(_plugin)
	{
		var panel = new Scroller(props);
		
		contents = new VBox({
			vmargin:1, hmargin:1, vpad:1,
			backgroundAlpha:0,
			binding:BIND_ALL
		});
		panel.setCanvas(contents);
	}
	
	function selectItemEvt(e)
	{
		selected = null;
		for(var i=0;i<items.length;++i)
		{
			if(items[i].panel == e.target)
			{
				selectItem(i);
				break;
			}
		}
	}
	
	function selectItem(index)
	{
		selected = null;
		if(index >= 0 && index < items.length) {
			selected = items[index];
			for(var i=0;i<items.length;++i)
				items[i].select(i == index);
		}
		activateBtn.activate(selected != null);
		return selected;
	}
	
	return {
		panel:panel,
		length:function() { return items.length; },
		selectedItem:function() { return selected; },
		selectItem:selectItem,
		
		addItem:function(item) {
			items.push(item);
			contents.addPane(item.panel);
			item.panel.addEventListener("onclick", selectItemEvt);
		},
		
		clearItems:function() {
			for(var i=0;i<items.length;++i)
				items[i].panel.removeEventListener("onclick", selectItemEvt);
			items = [];
			selected = null;
			activateBtn.activate(false);
			contents.removeAllPanes();
		},
		
		clearSelection:function() {
			for(var i=0;i<items.length;++i)
				items[i].select(false);
			activateBtn.activate(false);
		},
		
		itemIndex:function(item) {
		},
		
		editSelected:function() {
			if(selected)
				selected.edit();
		}
	};
}


/**
*
*/
function FavoriteItem(itemdef)
{
	var selected = false;
	
	with(_plugin)
	{
		var panel = new Frame({
			width:300, height:22,
			backgroundColor:0xffffff, backgroundAlpha:1,
			binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
		});

		var item = panel.addPane(new Input({
			x:4, y:1, width:100, height:20,
			text:itemdef.name, size:12,// bold:true,
			editable:false, fixedWidth:false, border:false
		}));
		item.addEventListener("onclick", function() {
			setFavoriteView();
			item.in_place_edit();
		});
		item.addEventListener("oncomplete", function() {
			itemdef.name = item.text;
			_plugin.database.execute("replace into favorites (id, name, data) VALUES(:id, :name, :data)", itemdef);
		});
	}
	
	function setFavoriteView() {
		var favdata = JSON.parse(itemdef.data);
		_plugin.camera.targetFeature = null;	// unset any camera targets
		_plugin.camera.properties = favdata;
		_plugin.viewOptions.properties = favdata.viewOptions;
		_mapLayers.setLayer(favdata.mapLayer);
	}
	
	return {
		panel:panel,
		itemID:function() { return itemdef.id; },
		selected:function() {return selected; },
		select:function(on) {
			selected = on;
			panel.backgroundColor = selected ? _selColor : 0xffffff;
			if(selected)
				setFavoriteView();
		},
		edit:function() {
			item.in_place_edit();
		}
	};
}
