/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/

/**
* FindControl
*
* 
* 
*/
function FindControl()
{	
	// The search function
	function doSearch() {
		results.removeAllPanes();
		placemarks.removeAllFeatures();
		
		if(search.text.length > 0)
		{
			var locations = _plugin.locationManager.findByName(search.text, 100);
			if(locations.length > 0)
			{
				for(var i=0;i<locations.length;++i)
				{
					placemarks.addFeature(new FoundFeature(locations[i]));
					results.addPane(new FoundLocation(locations[i]));
				}
			}
		}
	}
	
	function clearSearch() {
		results.removeAllPanes();
		placemarks.removeAllFeatures();
	}
	
	with(_plugin)
	try
	{
		var _focusfilter = [new GlowFilter(0x3f3fff, .75, 4, 4, 2, 3)];
		
		var icon = new _plugin.Icon({url:c_icon_base+"zoom_icon.png"});
		var placemarks = addFeature(new FeatureSet());
		var panel = new Pane({binding:BIND_ALL});
		
		// city pane
		var scrl = panel.addPane(new Scroller({
			x:1, y:26, width:-1, height:-26,
			backgroundAlpha:0.01,
			vscroll:true,
			binding:BIND_ALL
		}));
		
		var results = scrl.setCanvas(new VBox({
//			vmargin:1, hmargin:1,
			backgroundAlpha:0,
			binding:BIND_ALL
		}));
		
		
		// bottom bar container
		var btncont = panel.addPane(new Frame({
			y:0, height:26,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999],// backgroundAlpha:0.5,
			binding:BIND_LEFT|BIND_RIGHT
		}));
		
		
		// Search input text field
		var search_back = btncont.addPane(new Frame({
			x:10, y:4, width:-20, height:19,
			frameRadius:20, frameWidth:1, frameAlpha:0.85,
			binding:BIND_LEFT|BIND_RIGHT
		}));
		
		var search = search_back.addPane(
			new Input({
				x:16, y:1, height:18, width:-118,
//				x:62, y:2, width:-120, height:20,
				background:true, backgroundColor:0xffffff,
//				border:true, borderColor:0,
				size:12, bold:true,
				border:false, enableHilight:false,
				binding:BIND_LEFT|BIND_RIGHT
			})
		);
		search.addEventListener("oncommit", doSearch);
		search.addEventListener("onfocusin", function() { search_back.filters = _focusfilter; });
		search.addEventListener("onfocusout", function() { search_back.filters = []; });
		
		var mag = search_back.addPane(new Icon({x:4, y:4, skin:'greyMagnify'}));
		mag.addEventListener("onclick", function() {
			if(search.text.length > 0)
			{
				doSearch();
			}
			else
			{
				clearSearch();
				search.focus(true);
			}
		});
		
		var clrsearch = search_back.addPane(new Icon({x:-37, y:3, skin:'greyX', binding:BIND_TOP|BIND_RIGHT}));
		clrsearch.addEventListener("onclick", function() {
			results.removeAllPanes();
			placemarks.removeAllFeatures();
			search.text = "";
			search.focus(true);
		});

		clearSearch();
		
	}	
	catch(err)
	{
		alert(err.toString());
	}
	
	return {
		id:'eb_find',
		controlIcon:icon,
		controlPanel:panel,
		helpTip:_language['eb_find'],
		activate:function()			{ search.focus(true); },
		deactivate:function()		{ search.focus(false); }
	}
}

// Bootstrap
EarthBrowser.addModule('FindControl', FindControl);

/**
* FoundFeature
*
* Feature to be placed on the globe for the result. Clicking will zoom camera to location.
*/
function FoundFeature(loc)
{
	with(_plugin)
	{
		var cnt = loc.country;
		var state = loc.state;
		var st_name = state ? state.name : '';
		var cname = loc.name + (st_name.length > 0 ? (', ' + st_name) : '') + ', ' + (cnt ? cnt.name : '');
		
		var ftr = new Feature({name:cname});
		ftr.icon = new Icon({url:c_icon_base+"zoom_icon.png"});
		ftr.geometry = new Point(new LatLonAlt(loc.latitude, loc.longitude));
		ftr.addEventListener("onclick", function(e) {
			camera.properties = {longitude:loc.longitude, latitude:loc.latitude};
			_weather_wind.setLocation(loc);
		});
	}
	
	return ftr;
}


/**
* FoundLocation
*
* List element for the search result list. Clicking will zoom camera to location.
*/
function FoundLocation(loc)
{
	var backGrad = [0xffffff, 0xeeeeee], selGrad = [0xafafff, 0x8f8fcf];
	with(_plugin)
	{
		var cnt = loc.country;
		var state = loc.state;
		var st_name = state ? state.name : '';
		var cname = loc.name + (st_name.length > 0 ? (', ' + st_name) : '') + ', ' + (cnt ? cnt.name : '');
		
		var panel = new Frame({
			width:1000, height:24,
			backgroundColor:0xffffff, backgroundAlpha:0.5,
		});
		panel.addPane(new Icon({x:4, url:c_icon_base+"zoom_icon.png"}));
		panel.addPane(new Label({x:28, text:cname}));
		panel.addEventListener('onclick', function(e) {
			camera.properties = {longitude:loc.longitude, latitude:loc.latitude};
			_weather_wind.setLocation(loc);
		});
	}
	
	return panel;
}
