/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/

function OnEarthDailyPlanet()
{
	return new _plugin.WMSLayer({
		numZoomLevels:7, layers:'daily_planet',
		format:'image/jpeg', tileSize:512, expires:1.0,
		url:[
			'http://onearth.jpl.nasa.gov/wms.cgi',
			'http://oneearth.jpl.nasa.gov/wms.cgi',
			'http://wms.jpl.nasa.gov/wms.cgi',
			'http://mapus.jpl.nasa.gov/wms.cgi'
		]
	});
}

function layerItem(itemSet, itemdef)
{
	var radio = new _plugin.Radio({text:itemdef.name});
	radio.addEventListener("onclick", function() {
		for(var i=0;i<itemSet.length;++i)
			itemSet[i].setState(itemSet[i].name == itemdef.name);
	});
	
	function getState() {
		return radio.value;
	}
	
	function setState(on) {
		radio.value = on;
		if(on)
		{
			_plugin.layers = [itemdef.layer];
			_cache.store("_layer", itemdef.name);
		}
	}
	
	return {
		name:itemdef.name,
		icon:radio,
		getState:getState,
		setState:setState
	};
}

/**
* MapLayers
*/
function MapLayers()
{
	var items = [];
	var icon = new _plugin.Icon({url:c_icon_base+"layers.png"});
	var container = new _plugin.VBox({hmargin:8, vmargin:4, vpad:6, backgroundAlpha:0, binding:BIND_ALL});
	
	function addItem(itemdef) {
		var item = new layerItem(items, itemdef);
		container.addPane(item.icon);
		items.push(item);
	}
	
	addItem({name:"EarthBrowser", layer:new _plugin.Layer()});
	addItem({name:"OpenStreetMap", layer:new OpenStreetMapLayer()});
	addItem({name:"OnEarth Near Realtime Satellite", layer:new OnEarthDailyPlanet()});
	
//	addItem({name:"ESRI World Streetmap", layer:new ESRI_StreetMap_World_2D()});
		
//	addItem({name:"ESRI Shaded Relief", layer:ESRI_ShadedRelief_World_2D()});
//	addItem({name:"ESRI Physical World", layer:NPS_Physical_World_2D()});
//	addItem({name:"ESRI World Imagery", layer:ESRI_Imagery_World_2D()});
//	addItem({name:"NGS Topographical", layer:NGS_Topo_US_2D()});
	
	var panel = new _plugin.Scroller({
		backgroundAlpha:0.01, frameWidth:0, 
		vscroll:true, binding:BIND_ALL
	});
	panel.setCanvas(container);
	
	function getLayer() {
		for(var i=0;i<items.length;++i)
		{
			if(items[i].getState())
				return items[i].name;
		}
		return "EarthBrowser";
	}
	
	function setLayer(lid) {
		for(var i=0;i<items.length;++i)
			items[i].setState(items[i].name == lid);
	}
	
	setLayer(_cache.load("_layer", "EarthBrowser"))
	
	return {
		id:'map_layers',
		controlIcon:icon,
		controlPanel:panel,
		helpTip:_language['map_layers'],
		getLayer:getLayer,
		setLayer:setLayer
	}
}

// Bootstrap
EarthBrowser.addModule('MapLayers', MapLayers);
