/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/


var _featureWind = null;
var _defaultPmIconUrl = null;
var _mySelItems = null;

/**
* PersonalData
*/
function PersonalData()
{
	_defaultPmIconUrl = _cache.load("_defaultPmIconUrl", c_icon_base+"placemark_red.png");
	
	with(_plugin)
	try
	{
		var placemarks = _plugin.userFeatures;
		placemarks.editable = true;
//		var placemarks = addFeature(new FeatureSet({name:_language['personal_data'], open:true}));
		placemarks.addEventListener("onselect", function(e) {
			_mySelItems = placemarks.listView.selectedItems();
			gearBtn.activate(_mySelItems.length > 0);
		});
		
		var icon = new Icon({url:c_icon_base+"placemark_red.png"});
		var panel = new Pane({binding:BIND_ALL});
		
		// placemark pane
		var scrl = panel.addPane(new Scroller({
			x:1, y:1, width:-1, height:-22,
			backgroundAlpha:0.01,
			vscroll:true,
			binding:BIND_ALL
		}));
		scrl.addEventListener("onmousedown", function() {
			placemarks.listView.unselect_children();
			_mySelItems = placemarks.listView.selectedItems();
			gearBtn.activate(_mySelItems.length > 0);
		})
		
		scrl.setCanvas(placemarks.listView);
		
		// bottom bar container
		var btncont = panel.addPane(new Frame({
			y:-22, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999],// backgroundAlpha:0.5,
			binding:BIND_LEFT|BIND_RIGHT|BIND_BOTTOM
		}));
		
		// add button
		var addBtn = btncont.addPane(new Frame({
			width:24, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999], //backgroundAlpha:0.01,
			helpTip:_language['eb_new_pm']
		}));
		addBtn.addPane(new Frame({
			width:addBtn.width, height:addBtn.height,
			frameWidth:1,
			gradientColors:[0xffffff, 0xffffff], gradientAlphas:[0.5, 0.0],
		}));
		addBtn.addPane(new Icon({skin:'smallPlus', x:7, y:6}));
		addBtn.addEventListener("onclick", newPlacemark);
		
		// gear button
		var gearBtn = btncont.addPane(new Frame({
			x:24, width:32, height:22,
			frameWidth:1,
			gradientColors:[0xcccccc, 0x999999], //backgroundAlpha:0.01,
			helpTip:_language['eb_edit_pm']
		}));
		gearBtn.addPane(new Frame({
			width:gearBtn.width, height:gearBtn.height,
			frameWidth:1,
			gradientColors:[0xffffff, 0xffffff], gradientAlphas:[0.5, 0.0],
		}));
		gearBtn.addPane(new Icon({skin:'gear', x:7, y:6, scaleX:0.75, scaleY:0.75}));
		gearBtn.addPane(new Icon({skin:'downArrow', x:20, y:8, scaleX:0.75, scaleY:0.75}));
		gearBtn.activate(false);
		gearBtn.addEventListener("onclick", function() {
			var popup = new EditPMPopupMenu({x:gearBtn.globalX, y:gearBtn.globalY + gearBtn.height + 1});
			popup.addItem(new EditPMPopupMenuItem({text:_language['eb_edit_pm']}, editSelected));
			popup.addItem(new EditPMPopupMenuItem({text:_language['eb_del_pm']}, deleteSelected));
		});
	}	
	catch(err)
	{
		trace(err.toString());
	}
	
	function newPlacemark()
	{
		if(_featureWind)
		{
			playSound('buzz');
			return;
		}
	
		playSound('snickOn');
		
		with(_plugin)
		{
			var ftr = new Feature({id:getGUID(), name:"New Placemark"});
			ftr.icon = new Icon({url:_defaultPmIconUrl});
			ftr.geometry = new Point(new LatLonAlt(camera.latitude, camera.longitude));
			
			placemarks.addFeature(ftr);
			editFeature(ftr);
		}
	}
	
	function editFeature(ftr)
	{
		if(!_featureWind)
			_featureWind = new FeatureWind();
		_featureWind.setFeature(ftr);
	}
	
	function editSelected()
	{
		if(_mySelItems && _mySelItems.length > 0)
			editFeature(_mySelItems[0]);
		_mySelItems = placemarks.listView.selectedItems();
		gearBtn.activate(_mySelItems.length > 0);
	}
	
	function deleteSelected()
	{
		if(_mySelItems && _featureWind == null)
		{
			confirm("Delete " + _mySelItems.length + " placemarks?", function() {
				for(var i=0;i<_mySelItems.length;++i)
					placemarks.removeFeature(_mySelItems[i]);
				
				_mySelItems = placemarks.listView.selectedItems();
				gearBtn.activate(_mySelItems.length > 0);
			});
		}
	}
	
	function newFolder()
	{
		if(_featureWind)
		{
			playSound('buzz');
			return;
		}
	
		playSound('snickOn');
		
		with(_plugin)
		{
			var ftr = new FeatureSet({id:getGUID(), name:"New Folder"});
			placemarks.addFeature(ftr);
		}
	}
	
	function itemSelected()
	{
		return _mySelItems && _mySelItems.length > 0;
	}
	
	function keydown(e)
	{
		if(e.keyCode == air.Keyboard.BACKSPACE)
			deleteSelected();
	}
	
	function activate()
	{
		_plugin.addEventListener("onkeydown", keydown)
	}
	
	function deactivate()
	{
		_plugin.removeEventListener("onkeydown", keydown)
	}
	
	return {
		id:'personal_data',
		controlIcon:icon,
		controlPanel:panel,
		newPlacemark:newPlacemark,
		newFolder:newFolder,
		itemSelected:itemSelected,
		editSelected:editSelected,
		deleteSelected:deleteSelected,
		activate:activate,
		deactivate:deactivate,
		helpTip:_language['personal_data']
	}
}

// Bootstrap
EarthBrowser.addModule('PersonalData', PersonalData);

/**
* FeatureWind
*/
function FeatureWind()
{
	var feature = null;
	var zoomActive = false;
	
	with(_plugin)
	{
		var wind = addWindow(new Frame({
			x:200+_iwindoff, y:60+_iwindoff, width:300, height:180,
			draggable:true,
			frameRadius:10, frameWidth:1,
			gradientColors:[0xcccccc, 0xaaaaaa],// backgroundAlpha:0.5,
		}));
		wind.filters = _dropshadow;
		wind.addEventListener("onclose", closePlacemarkWindow);
		
		_iwindoff += 20;

		// close button
		var close = wind.addPane(new Icon({
			skin:'close',
			x:(os=="Mac") ? 4 : wind.width-18, y:4,
			binding:(os=="Mac") ? BIND_TOP|BIND_LEFT : BIND_TOP|BIND_RIGHT
		}));
		close.addEventListener("onclick", function(e) {
			removeWindow(wind);
		});

		// window resizer
		var resizer = wind.addPane(new Resizer({
			x:wind.width-13, y:wind.height-13,
			minWidth:300, minHeight:160,
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		resizer.target = wind;
		
		// window title
		var title = wind.addPane(new Label({
			x:0, y:2, width:wind.width,
			size:14, bold:true, align:"center",
			binding:BIND_TOP|BIND_LEFT|BIND_RIGHT|BIND_FIXED
		}));
		title.x = (wind.width - title.width) * 0.5;


		// Feature name
		var cname = wind.addPane(new Input({
			x:10, y:25, width:wind.width - 32, height:18,
			text:"", size:12, bold:false, fixedWidth:true, color:0,
			background:true, border:true, backgroundColor:0xffffff, borderColor:0, 
			binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
		}));
		cname.filters = _smdrop;
		cname.addEventListener("onchange", function() {
			if(feature)
				feature.name = cname.text;
			title.text = cname.text;
			title.x = (wind.width - title.width) * 0.5;
		});
		
		// latitude
		wind.addPane(new Label({
			x:10, y:50,
			id:"latitude", text:_language["latitude"], size:12, bold:true, 
			binding:BIND_LEFT
		}));

		var lat = wind.addPane(new Input({
			x:80, y:50, width:75, height:18,
			text:"",
			maxChars:10, restrict:'\\-0-9.',
			background:true, backgroundColor:0xffffff, 
			border:true, borderColor:0, 
			binding:BIND_LEFT
		}));
		lat.filters = _smdrop;
		lat.addEventListener("onchange", function() {
			if(feature)
				feature.geometry.latitude = lat.value;
		});
		
		// longitude
		wind.addPane(new Label({
			x:10, y:72,
			id:"longitude", text:_language["longitude"], size:12, bold:true, 
			binding:BIND_LEFT
		}));
		
		var lon = wind.addPane(new Input({
			x:80, y:72, width:75, height:18,
			text:"",
			maxChars:10, restrict:'\\-0-9.',
			background:true, backgroundColor:0xffffff, 
			border:true, borderColor:0, 
			binding:BIND_LEFT
		}));
			lon.filters = _smdrop;
		lon.addEventListener("onchange", function() {
			if(feature)
				feature.geometry.longitude = lon.value;
		});
		
		
		// icon popup
		var iconBtn = wind.addPane(new Frame({
			x:200, y:50, width:40, height:40,
			frameWidth:1, frameRadius:8, frameAlpha:.6,
			gradientColors:[0xeeeeee, 0xaaaaaa], backgroundAlpha:1
		}));
		iconBtn.filters = _dropshadow;
		var pmicon = iconBtn.addPane(new Icon({ url:_defaultPmIconUrl }));
		pmicon.x = (iconBtn.width - pmicon.width) * 0.5;
		pmicon.y = (iconBtn.height - pmicon.height) * 0.5;
		pmicon.addEventListener("onload", function() {
			pmicon.x = (iconBtn.width - pmicon.width) * 0.5;
			pmicon.y = (iconBtn.height - pmicon.height) * 0.5;
		});
		var darrow = iconBtn.addPane(new Icon({skin:'downArrow', x:28, y:20}));
		darrow.filters = _smdrop;
		iconBtn.addEventListener("onclick", function() {
			var iconPopup = new IconPopup({x:iconBtn.globalX, y:iconBtn.globalY}, function(url) {
				pmicon.url = url;
				if(feature)
					feature.icon.url = url;
			});
		});
		
		// Feature description
		wind.addPane(new Label({
			x:10, y:95,
			text:_language['description_html'], bold:true, color:0
		}))
		var cdescription = wind.addPane(new Input({
			x:10, y:115, width:wind.width - 32, height:55,
			text:"", size:12, bold:false, color:0, fixedWidth:true, multiline:true, 
			background:true, border:true, backgroundColor:0xffffff, borderColor:0, 
			binding:BIND_ALL
		}));
		cdescription.filters = _smdrop;
		cdescription.addEventListener("onchange", function() {
			if(feature)
				feature.description = cdescription.text;
		});
		
		// Zoom level settings

		// add city button
		var addCityBtn = wind.addPane(new Icon({
			id:"add_max_elev",
			skin:"small_plus",
			helpTip:_language['add_max_elev'],
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		addCityBtn.moveTo(wind.width_ - addCityBtn.width - 2, wind.height_ - addCityBtn.height - 14);
		addCityBtn.addEventListener("onclick", function() { setActiveZoom(!zoomActive); });

		// edit city button
		var editCityBtn = wind.addPane(new Icon({
			visible:false,
			id:"edit_max_elev",
			skin:"small_circle",
			helpTip:_language['edit_max_elev'],
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		editCityBtn.moveTo(wind.width_ - editCityBtn.width - 2, wind.height_ - editCityBtn.height - 14);
		editCityBtn.addEventListener("onclick", function() { setActiveZoom(!zoomActive); });

		// remove city button
		var removeCityBtn = wind.addPane(new Icon({
			id:"del_max_elev",
			skin:"small_minus",
			helpTip:_language['del_max_elev'],
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		removeCityBtn.moveTo(wind.width_ - removeCityBtn.width - 2, wind.height_ - addCityBtn.height - removeCityBtn.height - 18);
//		removeCityBtn.addEventListener("onclick", removeCity);
		removeCityBtn.activate(false);

		// Zoom level slider
		var zoomSlider = wind.addPane(new Slider({
			x:wind.width_-20, y:25,
			invert:true, vertical:true,
			binding:BIND_TOP|BIND_BOTTOM|BIND_RIGHT
		}));
		zoomSlider.setBack(new VBar({skin:"zoom_bar"}));
		zoomSlider.setThumb(new Icon({skin:"zoom_thumb"}));
		zoomSlider.moveTo(wind.width_-zoomSlider.width-5, 24);
		zoomSlider.resizeTo(zoomSlider.width, wind.height_ - 58 - removeCityBtn.height);
		camera.addEventListener("onzoom", function(e) {
			zoomSlider.value = camera.zoomLevel;
			if(zoomActive)
				updateZLevel();
		});
		zoomSlider.addEventListener("onslider", function(e) {
			camera.zoomLevel = zoomSlider.value;
			if(zoomActive)
				updateZLevel();
		});
		zoomSlider.activate(false);
		
		
	}
	
	function setFeature(ftr) {
		feature = ftr;
		if(!feature)
			return;
		
		title.text = feature.name;
		title.x = (wind.width - title.width) * 0.5;
		
		cname.text = feature.name;
		cname.selectAll();
		cname.focus(true);
		
		cdescription.text = feature.description;
		if(feature.geometry)
		{
			lon.text = Math.floor(feature.geometry.longitude * 100000)/100000;
			lat.text = Math.floor(feature.geometry.latitude * 100000)/100000;	
		}
		
		if(feature.icon)
		{
			feature.icon.addEventListener("onmousedown", startFtrDrag);	
			pmicon.url = feature.icon.url;
		}
	}

	function closePlacemarkWindow() {
//		popup.closePopup();
		if(feature && feature.icon)
			feature.icon.removeEventListener("onmousedown", startFtrDrag);
		_featureWind = null;
	}
	
	function startFtrDrag(e) {
		e.stopPropagation();
		_plugin.addEventListener("onmousemove", setFtrCoord);
		_plugin.addEventListener("onmouseup", stopFtrDrag);
	}
	
	function setFtrCoord(e) {
		e.stopPropagation();
		var coord = _plugin.camera.mouseCoord;
		if(coord)
		{
			coord.altitude = 0;
			lon.text = Math.floor(coord.longitude * 100000)/100000;
			lat.text = Math.floor(coord.latitude * 100000)/100000;
			feature.geometry.registrationPoint = coord;
		}
	}
	
	function stopFtrDrag(e) {
		e.stopPropagation();
		_plugin.removeEventListener("onmousemove", setFtrCoord);
		_plugin.removeEventListener("onmouseup", stopFtrDrag);
	}
	
	///
	function setActiveZoom(active)
	{
		zoomActive = active;
		_plugin.camera.removeEventListener("oncoordinate", resetZoomStatus);

		addCityBtn.skin = zoomActive ? 'small_x' : 'small_plus';
		editCityBtn.skin = zoomActive ? 'small_x' : 'small_circle';

		if(zoomActive)
		{
//			if(!centerOnLocation())
//				updateZLevel();
		}

		zoomSlider.activate(zoomActive && location != null);
		resetZoomButtonState();
	}

	///
	function resetZoomStatus()
	{
		_plugin.camera.removeEventListener("oncoordinate", resetZoomStatus);
		setActiveZoom(false);
	}

	///
	function resetZoomButtonState() {
//		var isSet = location ? (locationManager.getCityZoomLevel(location.id) >= 0) : false;
		var isSet = false;

		editCityBtn.visible = isSet;
		addCityBtn.visible = !isSet;
		removeCityBtn.activate(isSet && !zoomActive);
	}

	///
	function centerOnLocation() {

//		var zlevel = location ? locationManager.getCityZoomLevel(location.id) : -1;
		var zlevel = -1;
		var isSet = zlevel >= 0;

		// center on location coordinate
		_plugin.camera.properties = {
			speed:1.0,
			longitude:location.longitude,
			latitude:location.latitude,
			altitude:isSet ? _plugin.camera.levelToAltitude(zlevel) : _plugin.camera.altitude
		};

		// when camera moves centered coordinates, stop the zoom setting
		_plugin.camera.addEventListener("oncoordinate", resetZoomStatus);

		return isSet;
	}

	///
	function	updateZLevel()
	{
		if(location)
			_plugin.locationManager.setCityZoomLevel(location.id, zoomSlider.percentage * 1000.0);
	}

	return {
		setFeature:setFeature
	};
}

var c_pmicon_base = "http://globe.earthbrowser.com/placemarks/";
var _placemark_icons = [
	c_pmicon_base + 'placemark_blue.png',
	c_pmicon_base + 'placemark_cyan.png',
	c_pmicon_base + 'placemark_green.png',
	c_pmicon_base + 'placemark_pink.png',
	c_pmicon_base + 'placemark_purple.png',
	c_pmicon_base + 'placemark_red.png',
	c_pmicon_base + 'placemark_yellow.png',
	c_pmicon_base + 'placemark_orange.png',
	c_pmicon_base + 'palette-2_0_1.png',
	c_pmicon_base + 'palette-2_0_3.png',
	c_pmicon_base + 'palette-2_0_4.png',
	c_pmicon_base + 'palette-2_0_7.png',
	c_pmicon_base + 'palette-2_1_1.png',
	c_pmicon_base + 'palette-2_1_3.png',
	c_pmicon_base + 'palette-2_1_6.png',
	c_pmicon_base + 'palette-2_2_6.png',
	c_pmicon_base + 'palette-2_3_3.png',
	c_pmicon_base + 'palette-2_4_1.png',
	c_pmicon_base + 'palette-2_4_3.png',
	c_pmicon_base + 'palette-2_5_1.png',
	c_pmicon_base + 'palette-2_7_0.png',
	c_pmicon_base + 'palette-3_0_5.png',
	c_pmicon_base + 'palette-3_1_5.png',
	c_pmicon_base + 'palette-3_2_5.png',
	c_pmicon_base + 'palette-3_2_6.png',
	c_pmicon_base + 'palette-3_7_5.png',
	c_pmicon_base + 'palette-4_1_3.png',
	c_pmicon_base + 'palette-4_1_4.png',
	c_pmicon_base + 'palette-4_2_3.png',
	c_pmicon_base + 'palette-4_3_1.png',
	c_pmicon_base + 'palette-4_3_3.png',
	c_pmicon_base + 'palette-4_4_1.png',
	c_pmicon_base + 'palette-4_5_1.png',
	c_pmicon_base + 'palette-4_5_4.png',
	c_pmicon_base + 'palette-4_6_0.png',
	c_pmicon_base + 'palette-4_7_4.png',
	c_pmicon_base + 'palette-4_7_6.png',
	c_pmicon_base + 'palette-5_2_0.png',
	c_pmicon_base + 'palette-5_6_1.png',
	c_pmicon_base + 'palette-5_7_0.png',
];

function IconPopup(props, selFunc)
{
	var popup = null;
	
	function selectIcon(url) {
		return function() {
			_defaultPmIconUrl = url;
			selFunc(url);
			closePopup();
		}
	}

	function updatePopup(e) {
		e.target.removeEventListener("onload", updatePopup);
		if(popup)
			popup.fixSize();
	}

	function closePopup() {
		if(popup)
			_plugin.removeHover(popup);
		_plugin.removeEventListener("onmouseup", closePopup, true);
		popup = null;
	}

	with(_plugin)
	{
		props.vmargin = 4;
		props.hmargin = 4;
		props.frameWidth = 1;
		props.frameRadius = 6;
		props.gradientColors = [0xeeeeee, 0xbbbbbb];
		props.backgroundAlpha = 1;
			
		popup = new VBox(props);
		
		var rows = [];
		for(var i=0;i<8;++i)
			rows[i] = popup.addPane(new HBox({hpad:4, backgroundAlpha:0}))
		
		for(var i=0;i<_placemark_icons.length;++i)
		{
			var ico = rows[Math.min(7, Math.floor(i/8))].addPane(new Icon({url:_placemark_icons[i]}));
			ico.addEventListener("onload", function() { if(popup) popup.fixSize(); });
			ico.addEventListener("onclick", selectIcon(_placemark_icons[i]));
		}
		popup.fixSize();
		
		_plugin.addEventListener("onmouseup", closePopup, true);
		_plugin.addHover(popup);
	}

	return {
		panel:popup,
		closePopup:closePopup
	};
}




/**
*
*/
function EditPMPopupMenu(props)
{
//	x:button.globalX, y:button.globalY + button.height + 1
		
	with(_plugin)
	{
		props.vmargin = 4;
//		props.hmargin = 4;
		props.width = 100;
		props.height = 40;
		props.frameWidth = 1;
		props.frameRadius = 6;
		props.gradientColors = [0xeeeeee, 0xbbbbbb];
		props.backgroundAlpha = 1;
		
		var popup = addPane(new VBox(props));

		function closePopup() {
			_plugin.removePane(popup);
			_plugin.removeEventListener("onmouseup", closePopup, true);
			popup = null;
		}
		
		function addItem(item) {
			item.setMenu(this);
			popup.addPane(item.panel);
			popup.fixSize();
		}
		
		_plugin.addEventListener("onmouseup", closePopup, true);
	}
	
	return {
		panel:popup,
		addItem:addItem,
		closePopup:closePopup
	};
}

/**
*
*/
function EditPMPopupMenuItem(props, func)
{
	var menu = null;
	var panel = new _plugin.Frame({
		width:0, height:16,
		backgroundColor:0xffffff, backgroundAlpha:0.25,
		binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
	});
	panel.addPane(new _plugin.Label(props));
		
	panel.addEventListener('onclick', function(e) {
		menu.closePopup();
		func(e);
	});
	
	return {
		panel:panel,
		setMenu:function(m) {
			menu = m;
		}
	};
}
