/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/

var _satFeatureset = null, _findList = [];
var _fresults = null, _myresults = null;

///
/// SatelliteControl
///
///
function SatelliteControl()
{	
	var activeGrad = [0x777777, 0xaaaaaa], inactiveGrad = [0xeeeeee, 0xaaaaaa];
	var container, fscrl, findtab, myscrl, mysattab, search_back, fon = false;
	var satFindSpinner = null;
	
	function setSatFindPane(findon) {
		if(fon == findon)
			return;
			
		playSound('snickOn');
		fon = findon;
		if(findon) {
			container.removePane(myscrl);
			container.addPane(fscrl);
			fscrl.properties = {x:1, y:1, width:container.width_-1, height:container.height_};
			mysattab.gradientColors = inactiveGrad;
			findtab.gradientColors = activeGrad;
		}
		else {
			container.removePane(fscrl);
			container.addPane(myscrl);
			myscrl.properties = {x:1, y:1, width:container.width_-1, height:container.height_};
			mysattab.gradientColors = activeGrad;
			findtab.gradientColors = inactiveGrad;
		}
	}
	
	// The search function
	function doSearch() {
		clearSearch();
		
		if(search.text.length > 0)
		{
			setSatFindPane(true);
			satFindSpinner.start();
			var amf = new _plugin.AMFConnection('http://api.earthbrowser.com/gateway/');
			amf.addEventListener('onfail', function(e) {
				_plugin.trace("error");
				satFindSpinner.stop();
			});
			amf.addEventListener('onsuccess', function(e) {
				satFindSpinner.stop();
				var locations = amf.data;
//				_plugin.trace(JSON.stringify(amf.data, null, '\t'));
				if(locations.length > 0)
				{
					for(var i=0;i<locations.length;++i)
					{
						locations[i].visibility = false;
						var satitem = new SatelliteItem(locations[i]);
						satitem.setButton(AddSatelliteButton(satitem));
						_fresults.addPane(satitem.pane);
						_findList.push(satitem);
					}
				}
			});
			amf.callFunc('satellite.search', search.text);
		}
	}
	
	//
	function clearSearch() {
		_fresults.removeAllPanes();
		for(var i=0;i<_findList.length;++i)
			_findList[i].unloadKML();
		_findList = [];
	}
	
	with(_plugin)
	try
	{
		// create our table and add default satellites if table non-existant
		if(!database.tableExists("satellites"))
		{
			database.execute("create table if not exists satellites (nssdc text primary key, name text, alias text, visibility integer)");
			var dsats = [
				{nssdc:'1998-067A', name:'Zarya', alias:['International Space Station'], visibility:true},
				{nssdc:'1990-037B', name:'HST', alias:['Hubble Space Telescope'], visibility:true},
				{nssdc:'2008-029A', name:'GLAST', alias:['Fermi Gamma-ray Space Telescope'], visibility:true},
				{nssdc:'1989-089A', name:'COBE', alias:['Cosmic Background Explorer'], visibility:true},
				{nssdc:'2004-047A', name:'SWIFT', alias:['Swift Gamma Ray Burst Explorer'], visibility:true},
				{nssdc:'2003-017A', name:'GALEX', alias:['Galaxy Evolution Explorer'], visibility:true},
				{nssdc:'1991-063B', name:'UARS', alias:['Upper Atmosphere Research Satellite'], visibility:true},
				{nssdc:'1998-067BL', name:'1998-067BL', alias:['ISS Toolbag'], visibility:true},
			];
			for(var i=0;i<dsats.length;++i) {
				trace(dsats[i].alias);
				trace(JSON.stringify(dsats[i].alias));
				dsats[i].alias = JSON.stringify(dsats[i].alias);
				database.execute("replace into satellites (nssdc, name, alias, visibility) VALUES(:nssdc, :name, :alias, :visibility)", dsats[i]);
			}
		}
		
		var _focusfilter = [new GlowFilter(0x3f3fff, .75, 4, 4, 2, 3)];
		var icon = new _plugin.Icon({url:c_icon_base+"swift.png"});
		
		// container for satellites
		_satFeatureset = addFeature(new FeatureSet());
		_satFeatureset.visibility = _cache.load('_satellitesOn', true)
		
		var panel = new Pane({binding:BIND_ALL});
		
		
		//  bar container
		var btncont = panel.addPane(new Frame({
			y:0, height:30,
			frameWidth:1,
			gradientColors:[0xeeeeee, 0xcccccc],// backgroundAlpha:0.5,
			binding:BIND_LEFT|BIND_RIGHT
		}));
		
		// module pane container
		container = panel.addPane(new Frame({
			x:0, y:30, width:0, height:-32,
//			frameWidth:1,
			backgroundAlpha:0.75,
			binding:BIND_ALL
		}));
		
		///
		/// My Satellites section
		///
		
		// scroller pane
		myscrl = container.addPane(new Scroller({
			x:1, y:1, width:-1, height:-30,
			backgroundAlpha:0.01,
			vscroll:true,
			binding:BIND_ALL
		}));
		
		_myresults = myscrl.setCanvas(new VBox({
//			vmargin:1, hmargin:1,
			backgroundAlpha:0,
			binding:BIND_ALL
		}));
		
		mysattab = btncont.addPane(new _plugin.Frame({
			x:2, y:4, width:80, height:32, frameRadius:12, frameWidth:1,
//			helpTip:module.helpTip ? module.helpTip : null,
			gradientColors:activeGrad, backgroundAlpha:0.6,
			scrollHeight:26
		}));
		
		var satcheck = mysattab.addPane(new Checkbox({
			x:3, y:14, value:_satFeatureset.visibility
		}));
		satcheck.addEventListener("onupdate", function(e) {
			if(_satFeatureset.visibility != satcheck.value) {
				playSound('snickOn');
				_satFeatureset.visibility = satcheck.value;
				_cache.store("_satellitesOn", _satFeatureset.visibility);
			}
		});
		_satFeatureset.addEventListener("onupdate", function() {
			satcheck.value = _satFeatureset.visibility;
		});
			
		mysattab.addPane(new Label({
			x:18, y:4, bold:true, text:"Satellites"
		}));
		
		mysattab.addEventListener("onclick", function() {
			playSound('snickOn');
			setSatFindPane(false);
		});
		
		// load satellites
		var defs = database.execute("select nssdc, name, alias, visibility from satellites order by alias asc");
		if(defs)
		{
			for(var i=0;i<defs.length;++i)
			{
				defs[i].alias = JSON.parse(defs[i].alias);
				var satitem = new SatelliteItem(defs[i]);
				satitem.setButton(RemoveSatelliteButton(satitem));
				_myresults.addPane(satitem.pane);
				satitem.loadKML(false);
			}
		}
		
		///
		/// Find section
		///
		
		// scroller pane
		fscrl = new Scroller({
			x:1, y:1, width:0, height:0,
			backgroundAlpha:0.01,
			vscroll:true,
			binding:BIND_ALL
		});
		
		
		_fresults = fscrl.setCanvas(new VBox({
//			vmargin:1, hmargin:1,
			backgroundAlpha:0,
			binding:BIND_ALL
		}));
		
		
		findtab = btncont.addPane(new _plugin.Frame({
			x:85, y:4, width:70, height:32, frameRadius:12, frameWidth:1,
//			helpTip:module.helpTip ? module.helpTip : null,
			gradientColors:[0xeeeeee, 0xaaaaaa], backgroundAlpha:0.6,
			scrollHeight:26
		}));
		
		
		satFindSpinner = findtab.addPane(new Spinner({x:58, y:12, scaleX:0.6, scaleY:0.6}));
		satFindSpinner.stop();
		
		findtab.addPane(new Label({
			x:4, y:4, bold:true, text:"Search"
		}));
		findtab.addEventListener("onclick", function() {
			setSatFindPane(true);
		});
		
		// Search input text field
		search_back = btncont.addPane(new Frame({
			x:160, y:6, width:-165, height:19,
			frameRadius:20, frameWidth:1, frameAlpha:0.85,
			binding:BIND_LEFT|BIND_RIGHT
		}));
		
		var search = search_back.addPane(
			new Input({
				x:16, y:1, height:18, width:-188,
//				x:62, y:2, width:-120, height:20,
				background:true, backgroundColor:0xffffff,
//				border:true, borderColor:0,
				size:12, bold:true,
				border:false, enableHilight:false,
				binding:BIND_LEFT|BIND_RIGHT
			})
		);
		search.addEventListener("oncommit", doSearch);
		search.addEventListener("onfocusin", function() { search_back.filters = _focusfilter; });
		search.addEventListener("onfocusout", function() { search_back.filters = []; });
		
		var mag = search_back.addPane(new Icon({x:4, y:4, skin:'greyMagnify'}));
		mag.addEventListener("onclick", function() {
			if(search.text.length > 0)
			{
				doSearch();
			}
			else
			{
				clearSearch();
				search.focus(true);
			}
		});
		
		var clrsearch = search_back.addPane(new Icon({x:-182, y:3, skin:'greyX', binding:BIND_TOP|BIND_RIGHT}));
		clrsearch.addEventListener("onclick", function() {
			clearSearch();
			search.text = "";
			search.focus(true);
		});

		clearSearch();
		
	}	
	catch(err)
	{
		alert(err.toString());
	}
	
	return {
		id:'eb_satellite',
		controlIcon:icon,
		controlPanel:panel,
		helpTip:_language['eb_satellite'],
		activate:function()			{ search.focus(true); },
		deactivate:function()		{ search.focus(false); }
	}
}

// Bootstrap
EarthBrowser.addModule('SatelliteControl', SatelliteControl);

///
///
///
function setSat(sat)
{
	if(sat) {
		var lla = sat.geometry.registrationPoint;
		_plugin.camera.properties = {altitude:lla.altitude + 1e7, speed:0.005};
//		_plugin.camera.properties = {altitude:1e8, speed:0.005};
		_plugin.camera.targetFeature = sat;
	}
}

///
/// AddSatelliteButton
///
/// Button to add satellite to list.
///
function AddSatelliteButton(satitem)
{
	var defs = _plugin.database.execute("select nssdc from satellites where nssdc=:nssdc", {nssdc:satitem.satdef.nssdc});
	var has_sat = (defs && defs.length > 0);

	var icon = new _plugin.Icon({
		x:3, y:2,
		url:c_icon_base+"greenplus.png", alpha:has_sat ? 0.5 : 1.0,
		helpTip:_language['add_sat']		
	});
	
	if(!has_sat) {
		icon.addEventListener("onclick", function(e) {
//			icon.alpha = 0.5;
			satitem.satdef.visibility = true;
			_plugin.database.execute("replace into satellites (nssdc, name, alias, visibility) VALUES(:nssdc, :name, :alias, :visibility)", {
				nssdc:satitem.satdef.nssdc,
				name:satitem.satdef.name,
				alias:JSON.stringify(satitem.satdef.alias),
				visibility:satitem.satdef.visibility,
			});
			
			_fresults.removePane(satitem.pane);
			satitem.setButton(RemoveSatelliteButton(satitem));
			_myresults.addPane(satitem.pane);
			
			satitem.setOn(true);
		});
	}

	return icon;
}

///
/// RemoveSatelliteButton
///
/// Button to remove satellite to list.
///
function RemoveSatelliteButton(satitem)
{
	var icon = new _plugin.Icon({
		x:3, y:2, 
		url:c_icon_base+"redx.png",
		helpTip:_language['del']
	});
	icon.addEventListener("onclick", function(e) {
		confirm("Remove satellite?", function() {
			_plugin.database.execute("delete from satellites where nssdc = :nssdc", {nssdc:satitem.satdef.nssdc});
			_myresults.removePane(satitem.pane);
			satitem.unloadKML();
		});
	});
	return icon;
}


///
/// SatelliteItem
///
/// Element for a satellite in our list. Clicking will open info window for sat.
///
function SatelliteItem(sat)
{
	var button = null, kml = null, satkml = null;
	
	var pane = new _plugin.Frame({
		width:1000, height:24,
		backgroundColor:0xffffff, backgroundAlpha:0.5
	});
		
	var check = pane.addPane(new _plugin.Checkbox({
		x:25, y:11, value:sat.visibility
	}));
	check.addEventListener("onupdate", function(e) {
		setOn(check.value);
		saveStatus();
	});

	var zoomicn = pane.addPane(new _plugin.Icon({
		x:38, y:1, alpha:0.5,
		url:c_icon_base+"zoom_icon.png",
		helpTip:_language['view_sat']
	}));
	zoomicn.addEventListener("onclick", function(e) {
		if(satkml)
		{
			var satftr = satkml.findFeature(sat.nssdc);
			if(satftr)	setSat(satftr);
		}
	});

	var lbl = pane.addPane(new _plugin.Label({
		x:60, y:2, text:sat.alias[0] + ' (' + sat.name + ')',
		bold:true, underline:true, size:13, color:0x0000af
	}));
	lbl.addEventListener("onclick", function() {
		_plugin.addWindow(new _plugin.InfoWindow({
			x:200+_iwindoff, y:60+_iwindoff, width:400, height:300,
			text:"http://nssdc.gsfc.nasa.gov/nmc/spacecraftDisplay.do?id="+sat.nssdc,
			addressForce:true
		}));
		_iwindoff += 10;
	});
	
	///
	function saveStatus() {
		_plugin.database.execute("replace into satellites (nssdc, name, alias, visibility) VALUES(:nssdc, :name, :alias, :visibility)", {
			nssdc:sat.nssdc,
			name:sat.name,
			alias:JSON.stringify(sat.alias),
			visibility:sat.visibility,
		});
	}
	
	///
	function setOn(on) {
		zoomicn.alpha = on ? 1.0 : 0.5;
		sat.visibility = check.value = on;
		if(satkml)
			satkml.visibility = on;
		else if(on)
			loadKML(true);
	}
	
	///
	function loadKML(targetOnLoad) {
		if(satkml)
			return;
		
		zoomicn.alpha = 1.0;
		
		satkml = _satFeatureset.addFeature(_plugin.KMLFile({
			url:"http://api.earthbrowser.com/sat/nssdc_set/" + sat.nssdc + "/",
			refreshInterval:3600
		}));
		satkml.addEventListener("onclick", function(e) { setSat(e.target); });
		
		/*
		if(targetOnLoad)
		{
			satkml.addEventListener("onload", function() {
				var satftr = satkml.findFeature(sat.nssdc);
				if(satftr)	setSat(satftr);
			});
		}
*/
		return satkml;
	}
	
	///
	function unloadKML() {
		zoomicn.alpha = 0.5;
		if(satkml) {
			satkml.visibility = false;
			_satFeatureset.removeFeature(satkml);
			satkml = null;
		}
	}
	
	return {
		pane:pane,
		satdef:sat,
		setOn:setOn,
		setButton:function(btn) {
			if(button)	pane.removePane(button)
			button = btn;
			if(button)	pane.addPane(button);
		},
		loadKML:loadKML,
		unloadKML:unloadKML
	};
}

