/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/

/**
* TimeControl
*
* Control the camera zoom with a graphical slider with buttons
* 
*/
function TimeCtl()
{
	var timeAccel = 0.12;
	var gridView = false;
	var maxTimeOffset = 7, maxAnimSpeed = 100000;
	var panel = null, speedPopup = null, aspeedbtn = null;;
	
	function loadTimeState()
	{
		_plugin.clock.properties = _cache.load('_time_state', {
			clockMode:true,
			animating:false,
			clockStartOffset:0.25,
			clockEndOffset:0.001,
			animationSpeed:400
		});
	}
	
	function saveTimeState()
	{
		_cache.store('_time_state', {
			clockMode:_plugin.clock.clockMode,
			animating:_plugin.clock.animating,
			clockStartOffset:_plugin.clock.clockStartOffset,
			clockEndOffset:_plugin.clock.clockEndOffset,
			animationSpeed:_plugin.clock.animationSpeed
		});
	}
	
	function timeStr() {
		return gridView ? _plugin.clock.time.localeHourMinuteString() : _plugin.clock.time.localeString();
	}
	
	with(_plugin)
	{
		var backAlphaActive = 0.6, backAlphaInactive = 0.3;
		
		panel = new Pane({
			binding:BIND_LEFT|BIND_RIGHT
		});

		// Add a Slider control
		var slider = panel.addPane(new Slider({
			invert:false, vertical:false,
			binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
		}));
		
		// Set the slider back
		var back = slider.setBack(new Frame({
			height:18, width:-12,
			frameRadius:6, frameWidth:1, frameAlpha:0.8,
//			gradientColors:[0xd7ffce, 0x87d57e],
			gradientColors:[0x32B2A4, 0x22A294],
			backgroundAlpha:backAlphaInactive,
			binding:BIND_LEFT|BIND_RIGHT
		}));
		back.addPane(new Frame({
			x:2, y:1, width:back.width-3, height:14,
			frameRadius:8,
			gradientColors:[0xffffff, 0xffffff],
			gradientAlphas:[0.5, 0.0],
			binding:BIND_LEFT|BIND_RIGHT|BIND_TOP
		}));
		var ruler = back.addPane(new TimeRuler({y:6, width:back.width, binding:BIND_TOP|BIND_LEFT|BIND_RIGHT}));
		back.addEventListener("onmouseover", function() { back.backgroundAlpha = backAlphaActive; });
		back.addEventListener("onmouseout", function() { back.backgroundAlpha = backAlphaInactive; });
		
		// put the now thumb in front of the back, but under the thumb
		var nowThumb = back.addPane(new Icon({
			skin:"nowBar",
			id:'set_cur_time', helpTip:_language['set_cur_time'],
			x:ruler.nowPos - 3, y:-5,
			binding:BIND_TOP|BIND_LEFT
		}));
		nowThumb.addEventListener("onclick", function() {
			playSound('bump');
			clock.animating = false;
			clock.clockMode = true;
			saveTimeState();
		});
		
		// Set the Slider thumb
		var thumb = slider.setThumb(new Icon({y:6, skin:"time_arrow"}));
		thumb.addEventListener("onmouseover", function() { back.backgroundAlpha = backAlphaActive; });
		thumb.addEventListener("onmouseout", function() { back.backgroundAlpha = backAlphaInactive; });
		
		

		// Play/Pause buttons
		var play = panel.addPane(new Icon({
			id:'start_anim',
			skin:"play", x:-14, y:0,
			visible:!clock.animating,
			binding:BIND_TOP|BIND_RIGHT,
			helpTip:_language['start_anim']
		}));
		var pause = panel.addPane(new Icon({
			id:'pause_anim',
			skin:"pause", x:-14, y:0,
			visible:clock.animating,
			binding:BIND_TOP|BIND_RIGHT,
			helpTip:_language['pause_anim']
		}));
		play.addEventListener("onclick", function(e) {
			playSound('snickOn');
			clock.animating = true;
			saveTimeState();
		});
		pause.addEventListener("onclick", function(e) {
			playSound('snickOff');
			clock.animating = false;
			saveTimeState();
		});
		
		// Listen for the clock to change the slider value
		clock.addEventListener("onchange", function(e) {
			slider.value = clock.timePercentage;
			nowThumb.x = ruler.nowPos - nowThumb.width*0.5;
			timeText.text = timeStr();
		});
		
		// Set the play/pause buttons from our current state
		clock.addEventListener("onstatechange", function(e) {
			pause.visible = clock.animating;
			play.visible = !clock.animating;
			aslider.visible = clock.animating;
//			aspeedbtn.visible = clock.animating;
		});
		
		// listen for the slider value to change the zoom level
		slider.addEventListener("onslider", function(e) {
			clock.animating = clock.clockMode = false;
			clock.timePercentage = slider.value;
			timeText.text = timeStr();
		});
		
		slider.addEventListener("onsliderend", saveTimeState);
		
		
		// Current time text
		var timeText = panel.addPane(new Label({
//			x:-98, y:21,
			x:20, y:18,
			text:"", bold:true, color:0,
			fixedWidth:true,
//			align:"center",
			binding:BIND_TOP|BIND_LEFT|BIND_RIGHT|BIND_FIXED
		}));
		
		
		
		// Start offset control
		var minOffset = 1.0 / 3600.0;
		var offsetTimer = null;
		var bslider = panel.addPane(new Slider({
			x:2, y:25,
			invert:true, vertical:false,
			binding:BIND_TOP|BIND_LEFT
		}));

		var bback = bslider.setBack(new Frame({
			height:4, width:22,
			frameRadius:4,
//			frameWidth:1, frameAlpha:0.8, frameRadius:2,
			backgroundColor:0x444444,
//			backgroundAlpha:backAlphaInactive,
			binding:BIND_TOP|BIND_LEFT
		}));
		
		var bthumb = bslider.setThumb(new Icon({
			id:'set_anim_start',
			skin:"smDoubleArrow", y:-3,
			helpTip:_language['set_anim_start']
		}));
		bslider.value = 0.5;
		
		bslider.addEventListener("onsliderstart", function() {
			offsetTimer = new Timer(0.04);
			offsetTimer.addEventListener("ontimer", function() {
				var offset = clock.clockStartOffset;
				var delta = Math.max(0.01, offset) * timeAccel * (bslider.value - 0.5) * 2;
				offset = Math.max(minOffset, Math.min(offset + delta, maxTimeOffset));
				clock.clockStartOffset = offset;
				
				offsetTimer.restart();
			});
		});
		bslider.addEventListener("onsliderend", function() {
			bslider.value = 0.5;
			if(offsetTimer)
				offsetTimer.stop();
			offsetTimer = null;
			saveTimeState();
		});
		
		
		// End offset control
		var eslider = panel.addPane(new Slider({
			x:-46, y:25,
			invert:false, vertical:false,
			binding:BIND_TOP|BIND_RIGHT
		}));

		var eback = eslider.setBack(new Frame({
			height:4, width:22,
			frameRadius:4,
//			frameWidth:1, frameAlpha:0.8, frameRadius:2,
			backgroundColor:0x444444,
//			backgroundAlpha:backAlphaInactive,
			binding:BIND_TOP|BIND_LEFT
		}));
		var ethumb = eslider.setThumb(new Icon({
			id:'set_anim_end',
			skin:"smDoubleArrow", y:-3,
			helpTip:_language['set_anim_end']
		}));
		eslider.value = 0.5;
		
		eslider.addEventListener("onsliderstart", function() {
			offsetTimer = new Timer(0.04);
			offsetTimer.addEventListener("ontimer", function() {
				var offset = clock.clockEndOffset;
				var delta = Math.max(0.01, offset) * timeAccel * (eslider.value - 0.5) * 2;
				offset = Math.max(minOffset, Math.min(offset + delta, maxTimeOffset));
				clock.clockEndOffset = offset;
				
				offsetTimer.restart();
			});
		});
		eslider.addEventListener("onsliderend", function() {
			eslider.value = 0.5;
			if(offsetTimer)
				offsetTimer.stop();
			offsetTimer = null;
			saveTimeState();
		});
		
		
		// Animation Speed Control
		var aslider = panel.addPane(new Slider({
			x:-16, y:25,
			invert:false, vertical:false,
			binding:BIND_TOP|BIND_RIGHT
		}));

		var aback = aslider.setBack(new Frame({
			height:4, width:20,
			frameRadius:4,
			backgroundColor:0x444444,
			binding:BIND_TOP|BIND_LEFT
		}));
		var athumb = aslider.setThumb(new Icon({
			skin:"tachometer", y:-5,
			id:'anim_speed', helpTip:_language['anim_speed']
		}));
		aslider.value = 0.5;
		aslider.addEventListener("onsliderstart", function() {
			offsetTimer = new Timer(0.05);
			offsetTimer.addEventListener("ontimer", function() {
				
				var offset = clock.animationSpeed;
				var delta = Math.max(5, offset) * .2 * (aslider.value - 0.5);
				clock.animationSpeed = Math.max(5, Math.min(offset + delta, maxAnimSpeed));
				offsetTimer.restart();
			});
		});
		aslider.addEventListener("onsliderend", function() {
			aslider.value = 0.5;
			if(offsetTimer)
				offsetTimer.stop();
			offsetTimer = null;
			saveTimeState();
		});
		
		
		loadTimeState();
	}
	
	return {
		dockControl:panel,
		setGridView:function(on) {
			gridView = on;
			timeText.text = timeStr();
			timeText.x = back.x + (back.width_ - timeText.width_) / 2;
		}
	};
}

// Bootstrap
EarthBrowser.addModule('TimeControl', TimeCtl);
