/*
* Copyright (c) 2008 Matt Giger http://earthbrowser.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
* ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
* SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
* IN THE SOFTWARE.
*/

var c_wwind_width			= 356;
var c_wwind_height			= 155;
var c_toffset				= 22;
var c_icon_back_alpha		= 0.9;
var c_fcst_back_top			= 87;
var c_icon_back_left		= 4;
var c_icon_back_width		= 44;
var c_icon_back_height		= 63;
var c_icon_spacing			= 48;
var c_icon_left_offset		= 10;
var c_icon_top_offset		= 13;
var c_fcst_top_offset		= 35;
var c_hilo_top_offset		= 47;
var c_flag_width			= 84;
var c_flag_height			= 42;
var c_values_left			= 55;
var c_values_left2			= 160;

var c_day_names				= ['aday_sun','aday_mon','aday_tue','aday_wed','aday_thu','aday_fri','aday_sat'];

var c_flag_prefix			= "http://globe.earthbrowser.com/bflags/";
var c_wxicons_prefix		= "http://globe.earthbrowser.com/wxicons/";
var	c_wikipedia_prefix		= 'http://wikipedia.org/wiki/';

/**
* WeatherWind
*
* Show weather features
* 
*/
function WeatherWind()
{
	var base = null;
	var open = true;
	
	with(_plugin)
	{
		var location = null;
		var infowind = null;
		
		// add Weather icon skin
		addSkin(new Skin("weather", "wxicons.png", {
			wind:{ x:0, y:0, width:20, height:17 },
			humidity:{ x:20, y:0, width:14, height:18 },
			sunrise:{ x:0, y:20, width:24, height:24 },
			sunset:{ x:24, y:20, width:24, height:24 },
			pressure:{ x:34, y:0, width:16, height:16 },
			visibility:{ x:0, y:44, width:20, height:20 }
		}));

		var _back_dropshadow = [new DropShadowFilter(2, 45, 0, 0.7, 4, 4)];
		
		var wloc = _cache.load("_wx_window", {x:_plugin.width_ - c_wwind_width, y:0});
		
		// Create a draggable input frame
		var base =	new Frame({
			id:'_weather_window_',
			x:wloc.x, y:wloc.y, width:c_wwind_width, height:c_wwind_height,
			visible:false, draggable:true, bindEdge:true,
			frameRadius:16, frameWidth:1,
			backgroundAlpha:0.5, backgroundColor:0xaaaaaa//gradientColors:[0xeeeeee, 0x777777]
		});
		base.filters = _dropshadow;
		base.addEventListener("ondragend", function() {
			_cache.store("_wx_window", {x:base.x, y:base.y});
		});
		base.addEventListener("onclose", function() {
			open = false;
		});
		
		// white sheen
		base.addPane(new Frame({
			x:1, y:1, width:c_wwind_width-2, height:c_wwind_height/2,
			frameRadius:16, gradientAlphas:[0.5, 0], gradientColors:[0xffffff, 0xffffff]
		}));
		
		
		// City level zoom settings
		var zoomActive = false;
		
		///
		function setActiveZoom(active) {
			
			zoomActive = active;
			_plugin.camera.removeEventListener("oncoordinate", resetZoomStatus);
			
			addCityBtn.skin = zoomActive ? 'small_x' : 'small_plus';
			editCityBtn.skin = zoomActive ? 'small_x' : 'small_circle';
			
			if(zoomActive)
			{
				if(!centerOnLocation())
					updateZLevel();
			}

			zoomSlider.activate(zoomActive && location != null);
			resetZoomButtonState();
		}
		
		///
		function resetZoomStatus() {
			_plugin.camera.removeEventListener("oncoordinate", resetZoomStatus);
			setActiveZoom(false);
		}
		
		///
		function resetZoomButtonState() {
			var isSet = location ? (_plugin.locationManager.getCityZoomLevel(location.id) >= 0) : false;

			editCityBtn.visible = isSet;
			addCityBtn.visible = !isSet;
//			editCityBtn.state = addCityBtn.state = !zoomActive;
			removeCityBtn.activate(isSet && !zoomActive);
		}
		
		///
		function centerOnLocation() {
			
			var cam = _plugin.camera;
			var zlevel = location ? _plugin.locationManager.getCityZoomLevel(location.id) : -1;
			var isSet = zlevel >= 0;
			
			// center on location coordinate
			cam.properties = {
				speed:1.0,
				longitude:location.longitude,
				latitude:location.latitude,
				altitude:isSet ? cam.levelToAltitude(zlevel) : cam.altitude
			};
			
			// when camera moves centered coordinates, stop the zoom setting
			cam.addEventListener("oncoordinate", resetZoomStatus);
			
			return isSet;
		}
		
		///
		function	updateZLevel()
		{
			if(location)
				_plugin.locationManager.setCityZoomLevel(location.id, zoomSlider.percentage * 1000.0);
		}

		///
		function	removeCity()
		{
			if(location)
				_plugin.locationManager.removeCityZoomLevel(location.id);
			setActiveZoom();
		}
		
		
		// add city button
		var addCityBtn = base.addPane(new Icon({
			id:"add_max_elev",
			skin:"small_plus",
			helpTip:_language['add_max_elev'],
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		addCityBtn.moveTo(c_wwind_width - addCityBtn.width - 2, c_wwind_height - addCityBtn.height - 4);
		addCityBtn.addEventListener("onclick", function() { setActiveZoom(!zoomActive); });
		
		// edit city button
		var editCityBtn = base.addPane(new Icon({
			visible:false,
			id:"edit_max_elev",
			skin:"small_circle",
			helpTip:_language['edit_max_elev'],
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		editCityBtn.moveTo(c_wwind_width - editCityBtn.width - 2, c_wwind_height - editCityBtn.height - 4);
		editCityBtn.addEventListener("onclick", function() { setActiveZoom(!zoomActive); });
		
		// remove city button
		var removeCityBtn = base.addPane(new Icon({
			id:"del_max_elev",
			skin:"small_minus",
			helpTip:_language['del_max_elev'],
			binding:BIND_BOTTOM|BIND_RIGHT
		}));
		removeCityBtn.moveTo(c_wwind_width - removeCityBtn.width - 2, c_wwind_height - addCityBtn.height - removeCityBtn.height - 8);
		removeCityBtn.addEventListener("onclick", removeCity);
		removeCityBtn.activate(false);
		
		// Zoom level slider
		var zoomSlider = base.addPane(new Slider({
			x:c_wwind_width-20, y:25,
			invert:true, vertical:true
		}));
		zoomSlider.setBack(new VBar({skin:"zoom_bar"}));
		zoomSlider.setThumb(new Icon({skin:"zoom_thumb"}));
		zoomSlider.moveTo(c_wwind_width-zoomSlider.width-5, 24);
		zoomSlider.resizeTo(zoomSlider.width, c_wwind_height - 52 - removeCityBtn.height);
		camera.addEventListener("onzoom", function(e) {
			zoomSlider.value = camera.zoomLevel;
			if(zoomActive)
				updateZLevel();
		});
		zoomSlider.addEventListener("onslider", function(e) {
			camera.zoomLevel = zoomSlider.value;
			if(zoomActive)
				updateZLevel();
		});
		zoomSlider.activate(false);
		
		
		
		
		// city name, a click opens the Wikipedia entry
		var cityName = base.addPane(new Label({
			id:'wiki_city',
			x:0, y:2,
			bold:true, underline:true, size:13, color:0x0000af, text:"",
			helpTip:_language['wiki_city']
		}));
		cityName.addEventListener("onclick", function() {
			var name = location.name;
			var cnt = location.country;
			if(cnt && cnt.fips.toLowerCase() == 'us')
				name += '%2C_' + location.state.name;
			else
				name += '%2C_' + (cnt ? cnt.name : '');
			set_infowind(c_wikipedia_prefix + name);
		});
		
		// state name
		var stateName = base.addPane(new Label({
			id:'wiki_state',
			x:100, y:2,
			bold:true, underline:true, size:13, color:0x0000af, text:"",
			helpTip:_language['wiki_state']
		}));
		stateName.addEventListener("onclick", function(e) {
			var cnt = location.country;
			var state = location.state;
			var sname = state ? state.name : '';
			set_infowind(c_wikipedia_prefix + sname + (cnt ? ', ' + cnt.name : ''));
		});
		
		// country name
		var countryName = base.addPane(new Label({
			id:'wiki_country',
			x:base.width-64, y:20, useHandCursor:true,
			size:12, bold:true, underline:true, color:0x0000af, align:"right", text:"",
			helpTip:_language['wiki_country']
		}));
		countryName.addEventListener("onclick", function(e) {
			set_infowind(c_wikipedia_prefix + location.country.name);
		});
		
		// city time
		var cityTime = base.addPane(new Label({
			x:base.width - 100, y:2,
			size:12
		}));
		
		///
		function setCityTime() {
			if(location) {
				cityTime.text = location.getLocalTime().hourMinuteString();
				cityTime.x = c_wwind_width - cityTime.width - 10;
				cityTime.visible = true;
			}
			else
				cityTime.visible = false;
		}

		_plugin.addEventListener("onminutechange", setCityTime);

		
		// country flag
		// flag images are large, must scale them to fit
		var countryFlag = base.addPane(new Icon({
			x:base.width - c_flag_width - 30, y:40,
			width:c_flag_width, height:c_flag_height, useHandCursor:true,
			binding:BIND_TOP|BIND_RIGHT
		}));
		countryFlag.filters = _back_dropshadow;
		countryFlag.addEventListener("onload", function(e) {
			var cf = countryFlag;
			if(cf.width > c_flag_width) {
				cf.width = c_flag_width;
				cf.scaleY = cf.scaleX;
			}
			if(cf.height > c_flag_height) {
				cf.height = c_flag_height;
				cf.scaleX = cf.scaleY;
			}
			cf.x = base.width - cf.width - 30;
		});
		countryFlag.addEventListener("onclick", function(e) {
			set_infowind(c_wikipedia_prefix + location.country.name);
		});
		
		// close button
		var close = base.addPane(new Icon({
			skin:'close',
			x:(os=="Mac") ? 4 : base.width-18, y:4,
			binding:(os=="Mac") ? BIND_TOP|BIND_LEFT : BIND_TOP|BIND_RIGHT
		}));
		close.addEventListener("onclick", function(e) {
			base.visible = false;
			if(infowind)
			{
				infowind.dispose();
				infowind = null;
			}
		});
		
		
		
		// weather icon backdrop
		var wback = base.addPane(new Frame({
			x:c_icon_back_left, y:c_toffset,
			width:c_icon_back_width, height:c_icon_back_height, 
			alpha:c_icon_back_alpha, backgroundColor:0
		}));
		wback.filters = _back_dropshadow;
		
		
		// "current"
		base.addPane(new Label({
			x:c_icon_back_left-1, y:c_toffset, width:c_icon_back_width-1, height:9,
			id:'current_wx', text:_language['current_wx'], size:10, bold:true, color:0xffffff, 
			fixedWidth:false, binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
		}));
		
		// weather icon
		var wxicon = base.addPane(new Icon({
			x:c_icon_back_left + c_icon_left_offset, y:c_toffset+c_icon_top_offset
		}));
		
		// working spinner
		var working = base.addPane(new Spinner({
			x:c_icon_back_left + c_icon_left_offset + 12, y:c_toffset+c_icon_top_offset + 12,
			visible:false
		}));
		
		// current forecast
		var curfcst = base.addPane(new Label({
			x:c_icon_back_left-1, y:c_toffset+c_fcst_top_offset, width:c_icon_back_width, height:10,
			size:10, color:0xffffff, fixedWidth:false, binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
		}));
		
		// temperature
		var curtemp = base.addPane(new Label({
			x:c_icon_back_left-1, y:c_toffset+c_hilo_top_offset, width:c_icon_back_width, height:10,
			size:10, color:0xffffff, fixedWidth:false, binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
		}));
		
		// windspeed and direction
		var wind = base.addPane(new Label({
			id:'wind_spd_dir', helpTip:_language['wind_spd_dir'],
			x:c_values_left + 22, y:c_toffset, size:12
		}));
		
		// humidity
		var humidity = base.addPane(new Label({
			id:'humidity', helpTip:_language['humidity'],
			x:c_values_left + 22, y:c_toffset+23, size:12
		}));
		
		// sunrise
		var srise = base.addPane(new Label({
			id:'sunrise', helpTip:_language['sunrise'],
			x:c_values_left + 22, y:c_toffset+47, size:12
		}));
		
		// pressure
		var pressure = base.addPane(new Label({
			id:'pressure', helpTip:_language['pressure'],
			x:c_values_left2 + 22, y:c_toffset, size:12
		}));
		
		// visibility
		var visib = base.addPane(new Label({
			id:'visibility', helpTip:_language['visibility'],
			x:c_values_left2 + 22, y:c_toffset+23, size:12
		}));
		
		// sunset
		var sset = base.addPane(new Label({
			id:'sunset', helpTip:_language['sunset'],
			x:c_values_left2 + 22, y:c_toffset+47, size:12
		}));
		
		
		base.addPane(new Icon({skin:"weather:wind", x:c_values_left + 1, y:c_toffset}));
		base.addPane(new Icon({skin:"weather:humidity", x:c_values_left + 2, y:c_toffset+22}));
		base.addPane(new Icon({skin:"weather:sunrise", x:c_values_left + 2, y:c_toffset+39}));
		base.addPane(new Icon({skin:"weather:pressure", x:c_values_left2 + 2, y:c_toffset}));
		base.addPane(new Icon({skin:"weather:visibility", x:c_values_left2 + 0, y:c_toffset+22}));
		base.addPane(new Icon({skin:"weather:sunset", x:c_values_left2 -5, y:c_toffset+39}));
		
		var icons = [];
		var daynames = [];
		var fcst_txt = [];
		var hilos = [];
		for(var i=0; i<7; ++i)
		{
			var back = base.addPane(new Frame({
				x:c_icon_back_left + i*c_icon_spacing, y:c_fcst_back_top,
				width:c_icon_back_width, height:c_icon_back_height, 
				alpha:c_icon_back_alpha, backgroundColor:0
			}));
			back.filters = _back_dropshadow;
			
			var icon = base.addPane(new Icon({
				x:c_icon_back_left + c_icon_left_offset + i*c_icon_spacing, y:c_fcst_back_top + c_icon_top_offset
			}));
			icons.push(icon);
			
			var dname = base.addPane(new Label({
				x:c_icon_back_left + i*c_icon_spacing - 1, y:c_fcst_back_top - 2,
				width:c_icon_back_width, height:12,
				size:11, bold:true, color:0xffffff,
				fixedWidth:false, binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
			}));
			daynames.push(dname);
			
			var fcst = base.addPane(new Label({
				x:c_icon_back_left + i*c_icon_spacing - 1, y:c_fcst_back_top + c_fcst_top_offset,
				size:10, color:0xffffff, fixedWidth:false,
				width:c_icon_back_width, height:10, binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
			}));
			fcst_txt.push(fcst);
			
			var hilo = base.addPane(new Label({
				x:c_icon_back_left + i*c_icon_spacing - 1, y:c_fcst_back_top + c_hilo_top_offset,
				size:10, color:0xffffff, fixedWidth:false,
				width:c_icon_back_width, height:10, binding:BIND_TOP|BIND_LEFT|BIND_RIGHT
			}));
			hilos.push(hilo);
		}
		
		// set the current location
		function setLoc(loc)
		{
//			_plugin.trace('setLoc: ' + loc.id);
			if(!open)
				_plugin.addWindow(base);
			else
				_plugin.setTopWindow(base);	// bring us to the top
			
			location = loc;
			var cnt = loc.country;
			var state = loc.state;
			
			setCityTime();
			resetZoomStatus();
			
			base.visible = true;
			
			var st_name = state ? state.name : '';
			cityName.text = loc.name + (st_name.length > 0 ? ',' : '');
			stateName.text = st_name;
			
			cityName.x = (base.width - cityName.width - stateName.width) * 0.5;
			stateName.x = cityName.x + cityName.width + 2;

			countryName.text = cnt ? cnt.name : '';
			countryName.x = base.width - countryName.width - 30;
			
			if(cnt) {
				var furl = c_flag_prefix + cnt.fips.toLowerCase() + '.gif';
				if(furl != countryFlag.url)
					countryFlag.url = furl;	
			}
			
			// find the closest valid location for the forecast search
			var wxid = loc.id;
			if(wxid < 0) {
//				air.trace('no weather for loc');
				var closest = _plugin.locationManager.findClosestSet(loc.coordinate);
				for(var i=0;closest && i<closest.length;++i) {
					if(closest[i].id > 0) {
						wxid = closest[i].id
						break;
					}
				}
			}
			
			
			setWeather(null);	// clear fields
			if(wxid > 0)
			{
				working.start();
				var amf = new _plugin.AMFConnection('http://api.earthbrowser.com/gateway/');
				amf.addEventListener('onsuccess', function(e) { setWeather(amf.data); });
				amf.addEventListener('onfail', function(e) { working.stop(); });
				amf.callFunc('weather.status', wxid);
			}
		}
		
		// set the weather
		function setWeather(wxobj)
		{
//			air.trace(JSON.stringify(wxobj, null, '\t'));
			
			working.stop();
			if(wxobj && wxobj.observation && wxobj.forecast)
			{
				var metric = _plugin.viewOptions.metric;
				var time24 = _plugin.viewOptions.time24hr;
				var degree = metric ? "\xb0C" : "\xb0F";

				var temp = metric ? wxobj.observation.temp : celsius2F(wxobj.observation.temp);
				var windspd = metric ? wxobj.observation.wind : wxobj.observation.wind / 1.609
				var wdir = wxobj.observation.wind_dir;
				var pressureval = metric ? wxobj.observation.baro_pressure : wxobj.observation.baro_pressure / 33.865;
				var visd = metric ? wxobj.observation.visibility : wxobj.observation.visibility / 1.609;
				var speed_unit = metric ? _language['kmph'] : _language['miph'];
				var pressure_unit = metric ? _language['hectopascal'] : _language['inHg'];
				var distance_unit = metric ? _language['kilometer'] : _language['miles'];


				wxicon.url = c_wxicons_prefix + wxobj.observation.icon_id + '.png';
				curfcst.text = wxobj.observation.fcst;
				curtemp.text = (Math.floor(temp*10.0)/10.0).toString()+degree;
				wind.text = (windspd > 0) ? (Math.floor(windspd) + ' ' + speed_unit + ' ' + wdir) : _language['calm'];
				humidity.text = wxobj.observation.humidity + '%';
				pressure.text = Math.floor(pressureval*10)/10 + ' ' + pressure_unit;
				visib.text = Math.floor(visd) + ' ' + distance_unit;

				var sunrise = location.getSunrise(),sunset = location.getSunset();
				srise.text = sunrise ? utc_minute_str(sunrise, time24) : "--";
				sset.text = sunset ? utc_minute_str(sunset, time24) : "--";

				var len = wxobj.forecast.length;
				for(var i=0; i<len; ++i)
				{
					var fday = wxobj.forecast[i];
					var idx = fday.day_sequence-1;
					icons[idx].url = c_wxicons_prefix + fday.icon + '.png';
					daynames[idx].id = c_day_names[fday.day_of_week-1];
					daynames[idx].text = _language[c_day_names[fday.day_of_week-1]];
					fcst_txt[idx].text = fday.fcst;

					var high = metric ? fday.high_temp : celsius2F(fday.high_temp);
					var low = metric ? fday.low_temp : celsius2F(fday.low_temp);
					hilos[idx].text = Math.floor(high) + '/' + Math.floor(low) + degree;
				}
			}
			else
			{
				wxicon.url = "";
				curfcst.text = "";
				curtemp.text = "";
				wind.text = "";
				humidity.text = "";
				pressure.text = "";
				visib.text = "";
				srise.text = "--";
				sset.text = "--";

				for(var i=0; i<7; ++i)
				{
					icons[i].url = "";
					daynames[i].text = "";
					fcst_txt[i].text = "";
					hilos[i].text = "";
				}
			}
		}
		
		// create and populate the InfoWindow
		function set_infowind(url)
		{
			if(!infowind)
			{
				infowind = new _plugin.InfoWindow({x:200, y:60, width:200, height:200});
				_plugin.addWindow(infowind);
				infowind.addEventListener("onclose", function() {
//					_plugin.removeWindow(infowind);
					infowind = null;
				});
			}
			infowind.text = url;
		}
		
		function dispose()
		{
			if(infowind)
				_plugin.removeWindow(infowind);
			infowind = null;
		}
	}	
	
	return {
		wind:base,
		setLocation:setLoc,
		setLocationIcon:function(licon) {
			var loc = _plugin.locationManager.getLocation(licon.id);
			if(loc)
				setLoc(loc);
		}
	};
}

/**
*
*/
function celsius2F(temp)
{
	return temp * 1.8 + 32;
}

/**
*
*/
function utc_minute_str(date, time24)
{
	var hr = date.UTCHours;
	var mi = date.UTCMinutes;
	if(time24)
		return hr + ':' + ((mi < 10) ? '0' + mi : mi);
		
	var pm = (hr >= 12);
	var h = (hr > 12) ? hr - 12 : (hr == 0 ? 12 : hr);
	return h + ':' + ((mi < 10) ? '0' + mi : mi) + (pm ? 'pm' : 'am');
}

